# vim: set fileencoding=utf-8 :

import gtk
import gobject
import logging
import os
import pango
import time

from panelplugin.StatusIcon import StatusIcon

#----------------------------------------------------------------------------#
class BatteryIcon(StatusIcon):
#----------------------------------------------------------------------------#
    def __init__(self, parentObject = None, notificationAvailable = False):
        self.parentObject = parentObject
        self.notificationAvailable = notificationAvailable

        self.batstatus = ''
        self.capacity = None
        self.chargeFull = 0
        self.temp = None
        self.current = 0
        self.voltage = 0
        self.timeToEmpty = 0
        self.timeToFull = 0

        self.chargingRate = False
        self.batteryWindowActive = False

        StatusIcon.__init__(self, self.parentObject, 'battery')

        self.batteryPath = ''

        self.sysChargerType = None
        self.sysUsbLimit    = None

        if (self.parentObject.kernel == '2.6.24'):
            self.batteryPath = '/org/freesmartphone/Device/PowerSupply/bat'

            self.sysChargerType = "/sys/class/i2c-adapter/i2c-0/0-0073/charger_type"
            self.sysUsbLimit    = "/sys/class/i2c-adapter/i2c-0/0-0073/force_usb_limit_dangerous"
            
        elif (self.parentObject.kernel == '2.6.28'):
            self.batteryPath = '/org/freesmartphone/Device/PowerSupply/battery'

            self.sysChargerType = "/sys/class/i2c-adapter/i2c-0/0-0073/pcf50633-mbc/usb_curlim"
            self.sysUsbLimit    = "/sys/class/i2c-adapter/i2c-0/0-0073/pcf50633-mbc/force_usb_limit_dangerous"

        elif (self.parentObject.kernel == '2.6.29'):
            self.batteryPath = '/org/freesmartphone/Device/PowerSupply/battery'

            self.sysChargerType = None # currently not available on debian-krnl 2.6.29
            self.sysUsbLimit    = "/sys/class/power_supply/ac/device/usb_curlim"

        else:
            self.batteryPath = '/org/freesmartphone/Device/PowerSupply/3'

            self.sysChargerType = None # currently not available on debian-krnl 2.6.29
            self.sysUsbLimit    = "/sys/class/power_supply/ac/device/usb_curlim"

        self.dbusPowerStatusSignal = None
        self.dbusCapacitySignal = None

        self.dbusReInit()

    def dbusReInit(self, failure = False):
        self.batteryIface = None
        StatusIcon.dbusReInit(self, failure)

    def deferredDBusInit(self):
        if (StatusIcon.deferredDBusInit(self)):
            return True

        logging.info('entering init for battery')

        try:
            self.batteryIface = self.parentObject.tryGetInterface('org.freesmartphone.odeviced',
                                                                  self.batteryPath,
                                                                  'org.freesmartphone.Info')

            # try to get battery-info to ensure we connected to proper dbus-path
            self.batteryInfo = self.batteryIface.GetInfo()

        except Exception, e:
            logging.info("error on setting up and probing dbus-interface\n\t(%s)" % e)
            logging.info("assuming hardware is GTA01 ..")
            
            self.batteryPath = '/org/freesmartphone/Device/PowerSupply/bat_th_gta01'

            try:
                self.batteryIface = self.parentObject.tryGetInterface('org.freesmartphone.odeviced',
                                                                      self.batteryPath,
                                                                      'org.freesmartphone.Device.PowerSupply')

                self.sysChargerType = None # not existing on GTA01 ?
                self.sysUsbLimit    = None # not existing on GTA01 ?

                # try to get battery-info to ensure we connected to proper dbus-path
                self.batteryInfo = self.batteryIface.GetInfo()

            except Exception, e2:
                logging.error("error: no valid dbus-interface found ..\n\t(%s)" % e2)
                self.batteryIface = None            

        if (self.batteryIface):
            if (self.dbusPowerStatusSignal):
                try:
                    self.dbusPowerStatusSignal.remove()
                    self.dbusPowerStatusSignal = None
                except Exception, e:
                    logging.error("error while disconnecting from signal 'PowerStatus'\n\t(%s)" % e)

            if (not self.dbusPowerStatusSignal):
                self.dbusPowerStatusSignal = self.batteryIface.connect_to_signal('PowerStatus', self.updatePowerState)

            if (self.dbusCapacitySignal):
                try:
                    self.dbusCapacitySignal.remove()
                    self.dbusCapacitySignal = None
                except Exception, e:
                    logging.error("error while disconnecting from signal 'Capacity'\n\t(%s)" % e)

            if (not self.dbusCapacitySignal):
                self.dbusCapacitySignal = self.batteryIface.connect_to_signal('Capacity', self.updateCapacity)

            self.batteryInfo = self.updateBatteryInfo()

            self.updateCharger()

            self.updateIcon()

            self.startupDone = True

            return False

        logging.info('entering deferred init for battery')

        return True

    def updatePowerState(self, status = ''):
        if (self.batstatus != status):
            self.batstatus = status

            if (self.notificationAvailable & self.updateCharger()):
                self.commitNotification()

            self.updateIcon()

    def updateCapacity(self, capacity = None):
        if (self.capacity != capacity):
            self.capacity = capacity

            self.updateIcon()

    def updateBatteryInfo(self):
        iconUpdate = self.updateCharger()
        
        self.batteryInfo = {}

        if (self.batteryIface):
            try:
                self.batteryInfo = self.batteryIface.GetInfo()
            except Exception, e:
                logging.error('could not get batteryInfo\n\t(%s)' % e)

                self.dbusReInit(True)

        if ('status' in self.batteryInfo):
            status = self.batteryInfo[u'status']
            if (self.batstatus != status):
                self.batstatus = status
                iconUpdate = True

        if ('capacity' in self.batteryInfo):
            capacity = int(self.batteryInfo[u'capacity'])
            if (self.capacity != capacity):
                self.capacity = capacity
                iconUpdate = True

        if ('temp' in self.batteryInfo):
            temp = (float(self.batteryInfo[u'temp']) / 10)
            if (self.temp != temp):
                self.temp = temp

        if ('voltage_now' in self.batteryInfo):
            voltage = (float(self.batteryInfo[u'voltage_now']) / 1000000)
            if (self.voltage != voltage):
                self.voltage = voltage

        if ('current_now' in self.batteryInfo):
            current = -(float(self.batteryInfo[u'current_now']) / 1000)
            if (self.current != current):
                self.current = current
                iconUpdate = True

        if ('time_to_empty_now' in self.batteryInfo):
            timeToEmpty = int(self.batteryInfo[u'time_to_empty_now'])
            if (self.timeToEmpty != timeToEmpty):
                self.timeToEmpty = timeToEmpty

        if ('time_to_full_now' in self.batteryInfo):
            timeToFull = int(self.batteryInfo[u'time_to_full_now'])
            if (self.timeToFull != timeToFull):
                self.timeToFull = timeToFull

        if ('charge_full' in self.batteryInfo):
            chargeFull = int(self.batteryInfo[u'charge_full'])
            if (self.chargeFull != chargeFull):
                self.chargeFull = chargeFull

        if (iconUpdate):
            self.updateIcon()

    '''
    dbus.String(u'charge_full'): dbus.String(u'1145256', variant_level=1),
    dbus.String(u'health'): dbus.String(u'Good', variant_level=1),
    dbus.String(u'online'): dbus.String(u'1', variant_level=1),
    dbus.String(u'present'): dbus.String(u'1', variant_level=1)}, signature=dbus.Signature('sv'))
    '''

    def checkType(self):
        type = None

        if (self.sysChargerType):
            try:
                f = open(self.sysChargerType, 'r')
                type = f.readline().rstrip('\n')
                f.close()
            except Exception, e:
                logging.warning('assuming no charger connected\n\t(%s)' % e)

        currentRange = ''

        if (self.parentObject.kernel == '2.6.24'):
            try:
                currentRange = type.split(" ")[3]
            except:
                return False

            if (currentRange == '1A'):
                amperage = 1000
            elif (currentRange == '500mA'):
                amperage = 500
            elif (currentRange == '100mA'):
                amperage = 100
            else:
                amperage = False
        else:
            if (type in ['1000', '500', '100']):
                amperage = int(type)
            else:
                amperage = False

        return amperage

    def updateCharger(self):
        doNotify = False
        chargingRateNew = self.checkType()

        if (self.chargingRate != chargingRateNew):
            self.chargingRate = chargingRateNew
            doNotify = True

        return doNotify

    def getTime(self, s):
        years, s = divmod(s, 31556952)
        min, s = divmod(s, 60)
        h, min = divmod(min, 60)
        days, h = divmod(h, 24)

        ret = "" if years == 0 else "%sy " % years
        ret += "" if days == 0 else "%sd " % days
        ret += "%02dh " % h
        ret += "%02dm " % min
        ret += "%02ds" % s
        
        return ret

    def activate(self, widget, data = None):
        if (self.batteryWindowActive):
            self.menu.popup(None, None, None, 0, 0)
        else:
            self.batteryWindowMaker()

    def batteryWindowUpdate(self):
        self.updateBatteryInfo()

        self.batteryWindowCoulombCounterLabel.set_text("Coulomb-Counter: %s µA/h" % self.chargeFull)

        text = "Time to full: "

        if (self.timeToFull != 3932100):
            text += "%s" % self.getTime(self.timeToFull)
        else:
            text += "(no estimate)"

        self.batteryWindowTimeToFullLabel.set_text(text)

        text = "Time to empty: "

        if (self.timeToEmpty != 3932100):
            text += "%s" % self.getTime(self.timeToEmpty)
        else:
            text += "(no estimate)"

        self.batteryWindowTimeToEmptyLabel.set_text(text)

        self.batteryWindowCapacityLabel.set_text(("Capacity: %d%%" % self.capacity) if self.capacity else "Capacity: n/a")

        text = "Charging: "

        if (self.chargingRate):
            text +="%d mA" % self.chargingRate
        else:
            text += "%s" % self.batstatus

        self.batteryWindowChargingLabel.set_text(text)

        self.batteryWindowVoltageLabel.set_text("Voltage: %.2f V" % self.voltage)

        self.batteryWindowCurrentLabel.set_text("Current: %.0f mA" % self.current)

        self.batteryWindowTempLabel.set_text(("Temp: %.1f °C" % self.temp) if self.temp else "Temp: n/a")

        return self.batteryWindowActive

    def windowCallback(self, widget, data = None):
        if (data == 'close'):
            self.batteryWindowDestroyer(None)

        elif (data in ['100', '500', '1000']):
            if (self.doVibrationOnActivation):
                self.commitVibration(30)

            print "passo di qui"
            print "self.checkType()", self.checkType()
            print "self.sysUsbLimit", self.sysUsbLimit
            if (self.checkType() & self.sysUsbLimit):
                os.system("echo %s > %s" % (data, self.sysUsbLimit))

    def batteryWindowDestroyer(self, widget, data = None):
        self.batteryWindow.destroy()
        self.batteryWindowActive = False

    def batteryWindowDeleter(self, widget, event, data = None):
        return False

    def batteryWindowMaker(self):
        self.batteryWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.batteryWindow.set_title('Battery')
        self.batteryWindow.set_border_width(5)
        self.batteryWindow.connect("delete_event", self.batteryWindowDeleter)
        self.batteryWindow.connect("destroy" , self.batteryWindowDestroyer)
        self.batteryWindow.maximize()
        self.batteryWindow.show()

        self.batteryWindowFrame = gtk.Frame('Battery')
        self.batteryWindow.add(self.batteryWindowFrame)

        self.batteryWindowVBox = gtk.VBox(False, 0)
        self.batteryWindowVBox.set_border_width(5)

        self.batteryWindowHBox1 = gtk.HBox(True, 0)

        self.batteryWindowVBox2 = gtk.VBox(True, 0)

        self.batteryWindowCoulombCounterLabel = gtk.Label("Coulomb-Counter: (checking)")
        self.batteryWindowTimeToFullLabel = gtk.Label("Time to full: checking")
        self.batteryWindowTimeToEmptyLabel = gtk.Label("Time to empty: checking")
        self.batteryWindowCapacityLabel = gtk.Label("Capacity: checking")
        self.batteryWindowChargingLabel = gtk.Label("Charging: checking")
        self.batteryWindowVoltageLabel = gtk.Label("Voltage: checking")
        self.batteryWindowCurrentLabel = gtk.Label("Current: checking")
        self.batteryWindowTempLabel = gtk.Label("Temp: checking")

        self.batteryWindowVBox2.pack_start(self.batteryWindowCapacityLabel)
        self.batteryWindowVBox2.pack_start(self.batteryWindowChargingLabel)
        self.batteryWindowVBox2.pack_start(self.batteryWindowVoltageLabel)
        self.batteryWindowVBox2.pack_start(self.batteryWindowCurrentLabel)
        self.batteryWindowVBox2.pack_start(self.batteryWindowTempLabel)
        self.batteryWindowVBox2.pack_start(self.batteryWindowTimeToFullLabel)
        self.batteryWindowVBox2.pack_start(self.batteryWindowTimeToEmptyLabel)
        self.batteryWindowVBox2.pack_start(self.batteryWindowCoulombCounterLabel)

        gobject.timeout_add(2500, self.batteryWindowUpdate)
            
        self.batteryWindowHBox1.pack_start(self.batteryWindowVBox2)
        self.batteryWindowVBox.pack_start(self.batteryWindowHBox1)

        self.batteryWindowHBox2 = gtk.HBox(True, 0)

        self.batteryWindowChargeButton100 = gtk.Button(label="Charge at\n  100mA")
        self.batteryWindowChargeButton100.connect("pressed", self.windowCallback, "100")
        self.batteryWindowChargeButton100.set_sensitive(True if self.sysUsbLimit else False)
        self.batteryWindowChargeButton100.set_border_width(5)

        self.batteryWindowChargeButton500 = gtk.Button(label="Charge at\n  500mA")
        self.batteryWindowChargeButton500.connect("pressed", self.windowCallback, "500")
        self.batteryWindowChargeButton500.set_sensitive(True if self.sysUsbLimit else False)
        self.batteryWindowChargeButton500.set_border_width(5)

        self.batteryWindowChargeButton1000 = gtk.Button(label="Charge at\n  1000mA")
        self.batteryWindowChargeButton1000.connect("pressed", self.windowCallback, "1000")
        self.batteryWindowChargeButton1000.set_sensitive(True if self.sysUsbLimit else False)
        self.batteryWindowChargeButton1000.set_border_width(5)

        self.batteryWindowHBox2.pack_start(self.batteryWindowChargeButton100)
        self.batteryWindowHBox2.pack_start(self.batteryWindowChargeButton500)
        self.batteryWindowHBox2.pack_start(self.batteryWindowChargeButton1000)

        self.batteryWindowVBox.pack_start(self.batteryWindowHBox2)

        self.batteryWindowCloseButton = gtk.Button('Close')#, stock = gtk.STOCK_CLOSE)
#        self.batteryWindowCloseButton.child.child.get_children()[1].modify_font(pango.FontDescription("sans bold 30"))
        self.batteryWindowCloseButton.child.modify_font(pango.FontDescription("sans bold 30"))
        self.batteryWindowCloseButton.set_border_width(5)
        self.batteryWindowCloseButton.connect('clicked', self.windowCallback, 'close')

        self.batteryWindowVBox.pack_start(self.batteryWindowCloseButton)
        self.batteryWindowFrame.add(self.batteryWindowVBox)

        self.batteryWindow.show_all()

        self.batteryWindowActive = True

    def getIcon(self):
        bc = self.capacity
        file = ""

        if (self.chargingRate):
            bs = "_charging_%s" % self.chargingRate
        else:
            bs = ""

        if (bc >= 100):
            file = '%s/battery_100%s.png' % (self.parentObject.iconpath, bs)
        elif ((bc < 100) and (bc >= 90)):
            file = '%s/battery_090%s.png' % (self.parentObject.iconpath, bs)
        elif ((bc < 90) and (bc >= 80)):
            file = '%s/battery_080%s.png' % (self.parentObject.iconpath, bs)
        elif ((bc < 80) and (bc >= 70)):
            file = '%s/battery_070%s.png' % (self.parentObject.iconpath, bs)
        elif ((bc < 70) and (bc >= 60)):
            file = '%s/battery_060%s.png' % (self.parentObject.iconpath, bs)
        elif ((bc < 60) and (bc >= 50)):
            file = '%s/battery_050%s.png' % (self.parentObject.iconpath, bs)
        elif ((bc < 50) and (bc >= 40)):
            file = '%s/battery_040%s.png' % (self.parentObject.iconpath, bs)
        elif ((bc < 40) and (bc >= 30)):
            file = '%s/battery_030%s.png' % (self.parentObject.iconpath, bs)
        elif ((bc < 30) and (bc >= 20)):
            file = '%s/battery_020%s.png' % (self.parentObject.iconpath, bs)
        elif ((bc < 20) and (bc >= 10)):
            file = '%s/battery_010%s.png' % (self.parentObject.iconpath, bs)
        else:
#        elif ((bc < 10) and (bc >= 0)):
            file = '%s/battery_000%s.png' % (self.parentObject.iconpath, bs)
        
        return file

    def getTooltip(self):
        tooltip = "Battery-Capacity " + (("at %s%%" % self.capacity) if self.capacity else "n/a")

        if (self.chargingRate):
            tooltip += "\nCharging at %d mA" % self.chargingRate
        else:
            tooltip += "\n%s" % self.batstatus

        tooltip += "\nwith current: %.0f mA" % self.current

        tooltip += "\nremaining: "
        
        if (self.timeToFull != 3932100):
            tooltip += "%s" % self.getTime(self.timeToFull)
        elif (self.timeToEmpty != 3932100):
            tooltip += "%s" % self.getTime(self.timeToEmpty)
        else:
            tooltip += "(no estimate)"

        return tooltip

    def getNotificationText(self):
        notification = "Battery-Capacity " + ((" at %s%% %s" % (self.capacity, self.batstatus)) if self.capacity else "n/a")

        if (self.chargingRate):
            notification += "\nat %d mA" % self.chargingRate

        notification += "\nwith current: %.0f mA" % self.current

        return notification
