# vim: set fileencoding=utf-8 :

import gobject
import gtk
import logging
import pango

from panelplugin.StatusIcon import  StatusIcon

#----------------------------------------------------------------------------#
class BrightnessIcon(StatusIcon):
#----------------------------------------------------------------------------#
    def __init__(self, parentObject = None, notificationAvailable = False):
        self.parentObject = parentObject
        self.notificationAvailable = notificationAvailable

        self.brightnessWindowActive = False

        self.brightness = 100

        StatusIcon.__init__(self, self.parentObject, "brightness")

        self.dbusReInit()

    def dbusReInit(self, failure = False):
        self.deviceIface = None
        StatusIcon.dbusReInit(self, failure)

    def deferredDBusInit(self):
        if (StatusIcon.deferredDBusInit(self)):
            return True

        logging.info('entering init for brightness')

        self.deviceIface = self.parentObject.tryGetInterface('org.freesmartphone.odeviced',
                                                             '/org/freesmartphone/Device/Display/0',
                                                             'org.freesmartphone.Device.Display')

        if (self.deviceIface):
            self.startupDone = True

            return False

        logging.info('entering deferred init for brightness')

        return True

    def windowCallback(self, widget, data):
        if (data == 'close'):
            self.brightnessWindowActive = False
            self.brightnessWindow.destroy()
            self.brightnessWindow = None

        elif ((data == 'inc') and (self.brightness < 100)):
            if (self.doVibrationOnActivation):
                self.commitVibration(250)
            self.brightness += 5

        elif ((data == 'dec') and (self.brightness > 0)):
            if (self.doVibrationOnActivation):
                self.commitVibration(250)
            self.brightness -= 5

        if (self.brightness > 100):
            self.brightness = 100
        if (self.brightness < 0):
            self.brightness = 0

        self.brightnessAdjustement.set_value(self.brightness)

        try:
            self.deviceIface.SetBrightness(self.brightness)

        except Exception, e:
            logging.error('error while setting brightness to %s\n\t(%s)' % (self.brightness, e))
            self.dbusReInit(True)

    def activate(self, widget, data = None):
        if (self.brightnessWindowActive):
            self.menu.popup(None, None, None, 0, 0)
        else:
            self.brightnessWindowMaker()

    def brightnessWindowMaker(self):
        self.brightnessAdjustement = gtk.Adjustment(self.brightness, 0, 100, 1, 10, 1)
    
        self.brightnessWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.brightnessWindow.set_title('Brightness')
        self.brightnessWindow.set_border_width(10)
        self.brightnessWindow.maximize()
        self.brightnessWindow.show()

        self.brightnessWindowFrame = gtk.Frame('Brightness')
        self.brightnessWindow.add(self.brightnessWindowFrame)

        self.brightnessWindowVBox = gtk.VBox(False, 0)
        self.brightnessWindowVBox.set_border_width(5)
        self.brightnessWindowFrame.add(self.brightnessWindowVBox)

        self.bWHBox = gtk.HBox(False, 0)
                
        self.bWVBox2 = gtk.VBox(False, 0)
                
        self.bWIncButton = gtk.Button()
        self.bWIncButtonUpArrow = gtk.Arrow(gtk.ARROW_UP, gtk.SHADOW_OUT)
        self.bWIncButtonUpArrow.show()
        self.bWIncButton.add(self.bWIncButtonUpArrow)
        self.bWIncButton.set_border_width(5)
        self.bWIncButton.connect('pressed', self.windowCallback, 'inc')

        self.bWDecButton = gtk.Button()
        self.bWDecButtonDownArrow = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_OUT)
        self.bWDecButtonDownArrow.show()
        self.bWDecButton.add(self.bWDecButtonDownArrow)
        self.bWDecButton.set_border_width(5)
        self.bWDecButton.connect('pressed', self.windowCallback, 'dec')

        self.bWVBox2.pack_start(self.bWIncButton)
        self.bWVBox2.pack_start(self.bWDecButton)

        self.bWHBox.pack_start(self.bWVBox2)

        self.brightnessWindowVBox.pack_start(self.bWHBox)

        self.brightnessScale = gtk.VScale(self.brightnessAdjustement)
        self.brightnessScale.set_update_policy(gtk.UPDATE_CONTINUOUS)
        self.brightnessScale.set_inverted(True)
        self.brightnessScale.set_digits(0)
        self.brightnessScale.set_draw_value(True)
        self.brightnessScale.set_value_pos(gtk.POS_LEFT)
        self.brightnessScale.set_sensitive(False)

        self.bWHBox.pack_start(self.brightnessScale)

        self.brightnessWindowCloseButton = gtk.Button('Close')
#        self.brightnessWindowCloseButton.child.child.get_children()[1].modify_font(pango.FontDescription("sans bold 30"))
        self.brightnessWindowCloseButton.child.modify_font(pango.FontDescription("sans bold 30"))
        self.brightnessWindowCloseButton.set_border_width(5)
        self.brightnessWindowCloseButton.connect('clicked', self.windowCallback, 'close')

        self.brightnessWindowVBox.pack_start(self.brightnessWindowCloseButton, False)

        self.brightnessWindow.show_all()

        self.brightnessWindowActive = True

    def getTooltip(self):
        return "Brightness: %d%%" % self.brightness

    def getIcon(self):
        return "%s/brightness.png" % self.parentObject.iconpath

    def update(self):
        if (self.deviceIface):
            try:
                self.brightness = self.deviceIface.GetBrightness()
            except Exception, e:
                logging.error('error while getting brightness - assuming 100\%\n\t(%s)' % e)
                self.brightness = 100

                self.dbusReInit(True)

        if (self.brightnessWindowActive):
            self.brightnessAdjustement.set_value(self.brightness)

        self.updateIcon()

        return not self.parentObject.thread_terminate
