# vim: set fileencoding=utf-8 :

import array
import gtk
import gobject
import logging
import os
import pango
import signal
import subprocess
import time

from panelplugin.StatusIcon import StatusIcon

#----------------------------------------------------------------------------#
class Buttons(StatusIcon):
#----------------------------------------------------------------------------#
    def __init__(self, parentObject = None, notificationAvailable = False):
        self.parentObject = parentObject
        self.notificationAvailable = notificationAvailable

        self.shutdownExternal = False
        self.shutdownCommand = "shutdown -h now"
        self.confirmShutdown = False

        self.rebootExternal = False
        self.rebootCommand = "shutdown -r now"
        self.confirmReboot = False

        self.handleAux = True
        self.handleAuxLong = True
        self.handleAuxLongExternal = False
        self.handleAuxLongExternalCommand = ""
        self.handleAuxLongExternalPid = 0
        self.handleAuxLongTime = 1
        self.handleAuxLongOnlyStart = True
        self.handleAuxShort = True
        self.handleAuxShortExternal = True
        self.handleAuxShortExternalCommand = "/usr/bin/matchbox-keyboard"
        self.handleAuxShortExternalPid = 0

        self.handlePower = True
        self.handlePowerLong = False
        self.handlePowerLongExternal = False
        self.handlePowerLongExternalCommand = ""
        self.handlePowerLongExternalPid = 0
        self.handlePowerLongTime = 1
        self.handlePowerLongOnlyStart = True
        self.handlePowerShort = True
        self.handlePowerShortExternal = False
        self.handlePowerShortExternalCommand = ""
        self.handlePowerShortExternalPid = 0

        self.auxButtonCmds = [""] * 4
        self.auxButtonDescs = ["Aux 1", "Aux 2", "Aux 3", "Aux 4"]

        self.handleUsb = True

        self.handleHeadset = True
        self.handleHeadsetInsertion = False
        self.handleHeadsetInsertionCommand = ""
        self.handleHeadsetRemoval = False
        self.handleHeadsetRemovalCommand = ""

        self.auxButtonWindowActive = False
        self.powerButtonWindowActive = False
        self.lockWindowActive = False

        self.secondBuffer = 0

        self.auxWindowButton = 0
        self.powerWindowButton = 0

        StatusIcon.__init__(self, self.parentObject, "buttons")

        self.dbusSignal = None
        self.dbusReInit()

    def dbusReInit(self, failure = False):
        self.inputIface = None
        StatusIcon.dbusReInit(self, failure)

    def deferredDBusInit(self):
        if (StatusIcon.deferredDBusInit(self)):
            return True

        logging.info('entering init for buttons')

        self.inputIface = self.parentObject.tryGetInterface('org.freesmartphone.odeviced',
                                                            '/org/freesmartphone/Device/Input',
                                                            'org.freesmartphone.Device.Input')

        if (self.inputIface):
            if (self.dbusSignal):
                try:
                    self.dbusSignal.remove()
                    self.dbusSignal = None
                except Exception, e:
                    logging.warning("error while disconnecting from signal 'Event' \n\t(%s)" % e)

            if (not self.dbusSignal):
                self.dbusSignal = self.inputIface.connect_to_signal("Event", self.inputChanged)

            self.startupDone = True

            return False

        logging.warning('entering deferred init for buttons')

        return True

    def configReader(self):
        if (self.config.has_section(self.configName)):
            if (self.config.has_option(self.configName, "shutdownCommand")):
                self.shutdownCommand = self.config.get(self.configName, "shutdownCommand")

            if (self.config.has_option(self.configName, "confirmShutdown")):
                self.confirmShutdown = (self.config.get(self.configName, "confirmshutdown").lower() == "true")

            if (self.config.has_option(self.configName, "shutdownExternal")):
                self.shutdownExternal = (self.config.get(self.configName, "shutdownExternal").lower() == "true")

            if (self.config.has_option(self.configName, "rebootCommand")):
                self.rebootCommand = self.config.get(self.configName, "rebootCommand")

            if (self.config.has_option(self.configName, "confirmReboot")):
                self.confirmReboot = (self.config.get(self.configName, "confirmreboot").lower() == "true")

            if (self.config.has_option(self.configName, "rebootExternal")):
                self.rebootExternal = (self.config.get(self.configName, "rebootExternal").lower() == "true")

            if (self.config.has_option(self.configName, "handleAux")):
                self.handleAux = (self.config.get(self.configName, "handleAux").lower() == "true")

            if (self.config.has_option(self.configName, "handleAuxShort")):
                self.handleAuxShort = (self.config.get(self.configName, "handleAuxShort").lower() == "true")

            if (self.config.has_option(self.configName, "handleAuxShortExternal")):
                self.handleAuxShortExternal = (self.config.get(self.configName, "handleAuxShortExternal").lower() == "true")

            if (self.config.has_option(self.configName, "handleAuxShortExternalCommand")):
                self.handleAuxShortExternalCommand = self.config.get(self.configName, "handleAuxShortExternalCommand") 

            if (self.config.has_option(self.configName, "handleAuxLong")):
                self.handleAuxLong = (self.config.get(self.configName, "handleAuxLong").lower() == "true")

            if (self.config.has_option(self.configName, "handleAuxLongTime")):
                try: # someone could have messed up the config-file ..
                    self.handleAuxLongTime = int(self.config.get(self.configName, "handleAuxLongTime").lower())
                except:
                    self.handleAuxLongTime = 1

            if (self.config.has_option(self.configName, "handleAuxLongExternal")):
                self.handleAuxLongExternal = (self.config.get(self.configName, "handleAuxLongExternal").lower() == "true")

            if (self.config.has_option(self.configName, "handleAuxLongExternalCommand")):
                self.handleAuxLongExternalCommand = self.config.get(self.configName, "handleAuxLongExternalCommand")

            if (self.config.has_option(self.configName, "handleAuxLongOnlyStart")):
                self.handleAuxLongOnlyStart = (self.config.get(self.configName, "handleAuxLongOnlyStart").lower() == "true")

            if (self.config.has_option(self.configName, "handlePower")):
                self.handlePower = (self.config.get(self.configName, "handlePower").lower() == "true")

            if (self.config.has_option(self.configName, "handlePowerShort")):
                self.handlePowerShort = (self.config.get(self.configName, "handlePowerShort").lower() == "true")

            if (self.config.has_option(self.configName, "handlePowerShortExternal")):
                self.handlePowerShortExternal = (self.config.get(self.configName, "handlePowerShortExternal").lower() == "true")

            if (self.config.has_option(self.configName, "handlePowerShortExternalCommand")):
                self.handlePowerShortExternalCommand = self.config.get(self.configName, "handlePowerShortExternalCommand")

            if (self.config.has_option(self.configName, "handlePowerLong")):
                self.handlePowerLong = (self.config.get(self.configName, "handlePowerLong").lower() == "true")

            if (self.config.has_option(self.configName, "handlePowerLongTime")):
                try: # someone could have messed up the config-file ..
                    self.handlePowerLongTime = int(self.config.get(self.configName, "handlePowerLongTime").lower())
                except:
                    self.handlePowerLongTime = 1

            if (self.config.has_option(self.configName, "handlePowerLongExternal")):
                self.handlePowerLongExternal = (self.config.get(self.configName, "handlePowerLongExternal").lower() == "true")

            if (self.config.has_option(self.configName, "handlePowerLongExternalCommand")):
                self.handlePowerLongExternalCommand = self.config.get(self.configName, "handlePowerLongExternalCommand")

            if (self.config.has_option(self.configName, "handlePowerLongOnlyStart")):
                self.handlePowerLongOnlyStart = (self.config.get(self.configName, "handlePowerLongOnlyStart").lower() == "true")

            if (self.config.has_option(self.configName, "handleUsb")):
                self.handleUsb = (self.config.get(self.configName, "handleUsb").lower() == "true")

            if (self.config.has_option(self.configName, "handleHeadset")):
                self.handleHeadset = (self.config.get(self.configName, "handleHeadset").lower() == "true")

            if (self.config.has_option(self.configName, "handleHeadsetInsertion")):
                self.handleHeadsetInsertion = (self.config.get(self.configName, "handleHeadsetInsertion").lower() == "true")

            if (self.config.has_option(self.configName, "handleHeadsetInsertionCommand")):
                self.handleHeadsetInsertionCommand = (self.config.get(self.configName, "handleHeadsetInsertionCommand"))

            if (self.config.has_option(self.configName, "handleHeadsetRemoval")):
                self.handleHeadsetRemoval = (self.config.get(self.configName, "handleHeadsetRemoval").lower() == "true")

            if (self.config.has_option(self.configName, "handleHeadsetRemovalCommand")):
                self.handleHeadsetRemovalCommand = (self.config.get(self.configName, "handleHeadsetRemovalCommand"))

            for i in range(0, 4):
                if (self.config.has_option(self.configName, "auxbutton_%s_cmd" % i)):
                    self.auxButtonCmds[i] = self.config.get(self.configName, "auxbutton_%s_cmd" % i)

                if (self.config.has_option(self.configName, "auxbutton_%s_desc" % i)):
                    self.auxButtonDescs[i] = self.config.get(self.configName, "auxbutton_%s_desc" % i)

    def configWriter(self):
        if (not self.config.has_section(self.configName)):
            self.config.add_section(self.configName)

        if (self.shutdownCommand != ""):
            self.config.set(self.configName, "shutdownCommand", self.shutdownCommand)
        else:
            try:
                self.config.remove_option(self.configName, "shutdownCommand")
            except:
                pass

        if (self.confirmShutdown):
            self.config.set(self.configName, "confirmShutdown", "True")
        else:
            try:
                self.config.remove_option(self.configName, "confirmShutdown")
            except:
                pass

        if (self.shutdownExternal):
            self.config.set(self.configName, "shutdownExternal", "True")
        else:
            try:
                self.config.remove_option(self.configName, "shutdownExternal")
            except:
                pass

        if (self.rebootCommand != ""):
            self.config.set(self.configName, "rebootCommand", self.rebootCommand)
        else:
            try:
                self.config.remove_option(self.configName, "rebootCommand")
            except:
                pass

        if (self.confirmReboot):
            self.config.set(self.configName, "confirmReboot", "True")
        else:
            try:
                self.config.remove_option(self.configName, "confirmReboot")
            except:
                pass

        if (self.rebootExternal):
            self.config.set(self.configName, "rebootExternal", "True")
        else:
            try:
                self.config.remove_option(self.configName, "rebootExternal")
            except:
                pass

        self.config.set(self.configName, "handleAux", "True" if self.handleAux else "False")

        self.config.set(self.configName, "handleAuxLong", "True" if self.handleAuxLong else "False")
        self.config.set(self.configName, "handleAuxLongTime", "%s" % self.handleAuxLongTime)
        self.config.set(self.configName, "handleAuxLongExternal", "True" if self.handleAuxLongExternal else "False")

        if (self.handleAuxLongExternalCommand != ""):
            self.config.set(self.configName, "handleAuxLongExternalCommand", self.handleAuxLongExternalCommand)
        else:
            try:
                self.config.remove_option(self.configName, "handleAuxLongExternalCommand")
                self.config.remove_option(self.configName, "handleAuxLongOnlyStart")
            except:
                pass

        self.config.set(self.configName, "handleAuxShort", "True" if self.handleAuxShort else "False")
        self.config.set(self.configName, "handleAuxShortExternal", "True" if self.handleAuxShortExternal else "False")

        if (self.handleAuxShortExternalCommand != ""):
            self.config.set(self.configName, "handleAuxShortExternalCommand", self.handleAuxShortExternalCommand)
        else:
            try:
                self.config.remove_option(self.configName, "handleAuxShortExternalCommand")
            except:
                pass

        self.config.set(self.configName, "handlePower", "True" if self.handlePower else "False")

        self.config.set(self.configName, "handlePowerLong", "True" if self.handlePowerLong else "False")
        self.config.set(self.configName, "handlePowerLongTime", "%s" % self.handlePowerLongTime)
        self.config.set(self.configName, "handlePowerLongExternal", "True" if self.handlePowerLongExternal else "False")

        if (self.handlePowerLongExternalCommand != ""):
            self.config.set(self.configName, "handlePowerLongExternalCommand", self.handlePowerLongExternalCommand)
        else:
            try:
                self.config.remove_option(self.configName, "handlePowerLongExternalCommand")
                self.config.remove_option(self.configName, "handlePowerLongOnlyStart")
            except:
                pass

        self.config.set(self.configName, "handlePowerShort", "True" if self.handlePowerShort else "False")
        self.config.set(self.configName, "handlePowerShortExternal", "True" if self.handlePowerShortExternal else "False")

        if (self.handlePowerShortExternalCommand != ""):
            self.config.set(self.configName, "handlePowerShortExternalCommand", self.handlePowerShortExternalCommand)
        else:
            try:
                self.config.remove_option(self.configName, "handlePowerShortExternalCommand")
            except:
                pass        

        self.config.set(self.configName, "handleUsb", "True" if self.handleUsb else "False")

        self.config.set(self.configName, "handleHeadset", "True" if self.handleHeadset else "False")

        self.config.set(self.configName, "handleHeadsetInsertion", "True" if self.handleHeadsetInsertion else "False")

        if (self.handleHeadsetInsertionCommand != ""):
            self.config.set(self.configName, "handleHeadsetInsertionCommand", self.handleHeadsetInsertionCommand)
        else:
            try:
                self.config.remove_option(self.configName, "handleHeadsetInsertionCommand")
            except:
                pass        
            
        self.config.set(self.configName, "handleHeadsetRemoval", "True" if self.handleHeadsetRemoval else "False")

        if (self.handleHeadsetRemovalCommand != ""):
            self.config.set(self.configName, "handleHeadsetRemovalCommand", self.handleHeadsetRemovalCommand)
        else:
            try:
                self.config.remove_option(self.configName, "handleHeadsetRemovalCommand")
            except:
                pass        

        for i in range(0, 4):
            if (self.auxButtonCmds[i] != ""):
                self.config.set(self.configName, "auxbutton_%s_cmd" % i, self.auxButtonCmds[i])
                self.config.set(self.configName, "auxbutton_%s_desc" % i, self.auxButtonDescs[i])

    def inputChanged(self, name = "", action = "", seconds = 0):
        self.secondBuffer = seconds

        if (self.lockWindowActive):
            if ((name in ["AUX", "POWER"]) & (action == "released")):
                self.showLockWindow()
        else:
            if (not self.parentObject.buttonHandler(name, action, seconds)):
                if ((name == "AUX") & self.handleAux):
                    if (self.auxButtonWindowActive):
                        if (action == "pressed"):
                            self.auxWindowButton = 0 if (self.auxWindowButton == len(self.auxButtonWindowButtons) - 1) else (self.auxWindowButton + 1)

                            (self.auxButtonWindowButtons[self.auxWindowButton]).grab_focus()

                            for button in self.auxButtonWindowButtons:
                                button.set_state(gtk.STATE_NORMAL)

                            (self.auxButtonWindowButtons[self.auxWindowButton]).set_state(gtk.STATE_SELECTED)

                    elif (self.powerButtonWindowActive):
                        if (action == "pressed"):
                            self.powerWindowButton = 0 if (self.powerWindowButton == len(self.powerButtonWindowButtons) - 1) else (self.powerWindowButton + 1)
    
                            (self.powerButtonWindowButtons[self.powerWindowButton]).grab_focus()

                            for button in self.powerButtonWindowButtons:
                                button.set_state(gtk.STATE_NORMAL)

                            (self.powerButtonWindowButtons[self.powerWindowButton]).set_state(gtk.STATE_SELECTED)

                    else:
                        if (action == "pressed"):
                            pass
                        elif (action == "held"):
                            if (self.handleAuxLong & (self.secondBuffer == self.handleAuxLongTime)):
                                # long aux
                                if (self.handleAuxLongExternal):
                                    if ((self.handleAuxLongExternalPid != 0) & (not self.handleAuxLongOnlyStart)):
                                        try:
                                            os.kill(self.handleAuxLongExternalPid, signal.SIGTERM)
                                            os.waitpid(self.handleAuxLongExternalPid, 0)
                                        except:
                                            logging.error("cannot kill aux long pid (%s) , died?" % self.handleAuxLongExternalPid)

                                        self.handleAuxLongExternalPid = 0

                                    else:
                                        self.handleAuxLongExternalPid = subprocess.Popen(self.handleAuxLongExternalCommand, shell = True).pid
                                else:
                                    self.auxWindowMaker()

                        elif (action == "released"):
                            if (self.secondBuffer == 0):
                                # short aux
                                if (self.handleAuxShort):
                                    if (self.handleAuxShortExternal):
                                        if (self.handleAuxShortExternalPid != 0):
                                            try:
                                                os.kill(self.handleAuxShortExternalPid, signal.SIGTERM)
                                                os.waitpid(self.handleAuxShortExternalPid, 0)
                                            except:
                                                logging.error("cannot kill aux short pid (%s), died?" % self.handleAuxShortExternalPid)
                                            self.handleAuxShortExternalPid = 0
                                        else:
                                            self.handleAuxShortExternalPid = subprocess.Popen(self.handleAuxShortExternalCommand, shell = True).pid
                                    else: # do not handle aux-short external
                                        pass
                            else:
                                pass

                            self.secondBuffer = 0

                elif ((name == "POWER") & self.handlePower):
                    if (self.auxButtonWindowActive):
                        if (action == "released"):
                            self.auxButtonWindowButtons[self.auxWindowButton].activate()

                    elif (self.powerButtonWindowActive):
                        if (action == "released"):
                            self.powerButtonWindowButtons[self.powerWindowButton].activate()

                    else:
                        if (action == "pressed"):
                            pass
                        elif (action == "held"):
                            if (self.handlePowerLong & (self.secondBuffer == self.handlePowerLongTime)):
                                # long power
                                if (self.handlePowerLongExternal):
                                    if (self.handlePowerLongExternalPid != 0):
                                        try:
                                            os.kill(self.handlePowerLongExternalPid, signal.SIGTERM)
                                            os.waitpid(self.handlePowerLongExternalPid, 0)
                                        except:
                                            logging.error("cannot kill aux long pid (%s), died?" % self.handlePowerLongExternalPid)

                                        self.handlePowerLongExternalPid = 0
                                    else:
                                        self.handlePowerLongExternalPid = subprocess.Popen(self.handlePowerLongExternalCommand, shell = True).pid
                                else:
                                    self.parentObject.signalTo('screenlock', lockScreen = True)

                        elif (action == "released"):
                            if (self.secondBuffer == 0):
                                # short power
                                if (self.handlePowerShort):
                                    if (self.handlePowerShortExternal):
                                        if ((self.handlePowerShortExternalPid != 0) & (not self.handlePowerLongOnlyStart)):
                                            try:
                                                os.kill(self.handlePowerShortExternalPid, signal.SIGTERM)
                                                os.waitpid(self.handlePowerShortExternalPid, 0)
                                            except:
                                                logging.error("cannot kill aux short pid (%s), died?" % self.handlePowerShortExternalPid)

                                            self.handlePowerShortExternalPid = 0
                                        else:
                                            self.handlePowerShortExternalPid = subprocess.Popen(self.handlePowerLongExternalCommand, shell = True).pid
                                    else:
                                        self.powerButtonWindowMaker()

                            self.secondBuffer = 0

                elif ((name == "USB") & self.handleUsb):
                    if (action == "pressed"):
                        pass
                    elif (action == "released"):
                        pass

                    self.parentObject.signalTo('battery', update = True)

                elif ((name == "HEADSET") & self.handleHeadset):
                    if (action == "pressed"):
                        if (self.handleHeadsetInsertion & (self.handleHeadsetInsertionCommand != "")):
                            os.system("%s &" % self.handleHeadsetInsertionCommand)

                    elif (action == "released"):
                        if (self.handleHeadsetRemoval & (self.handleHeadsetRemovalCommand != "")):
                            os.system("%s &" % self.handleHeadsetRemovalCommand)

    def auxWindowCallback(self, widget, data = None):
        windowClose = False

        for i in range(0, len(self.auxButtonCmds)):
            if (data == "auxbutton_%s" % i):
                os.system("%s >/dev/null 2>&1 &" % self.auxButtonCmds[i])
                windowClose = True

        if (data == 'close'):
            windowClose = True

        if (windowClose):
            self.auxWindowDestroyer(None)

    def auxWindowDestroyer(self, widget, data = None):
        self.auxButtonWindow.destroy()
        self.auxButtonWindowActive = False

    def auxWindowDeleter(self, widget, event, data = None):
        return False

    def auxWindowMaker(self):
        if (self.auxButtonWindowActive):
            self.auxWindowDestroyer(None)
        else:
            self.auxWindowButton = 0

            self.auxButtonWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
            self.auxButtonWindow.set_title('Aux')
            self.auxButtonWindow.set_border_width(5)
            self.auxButtonWindow.connect("delete_event", self.auxWindowDeleter)
            self.auxButtonWindow.connect("destroy", self.auxWindowDestroyer)
            self.auxButtonWindow.maximize()

            self.auxButtonWindowFrame = gtk.Frame('Aux (AUX cycles through buttons, POWER executes)')
            self.auxButtonWindow.add(self.auxButtonWindowFrame)

            self.auxButtonWindowVBox = gtk.VBox(False, 0)
            self.auxButtonWindowFrame.add(self.auxButtonWindowVBox)

            self.auxButtonWindowButtons = [] #[None] * 5

            for i in range(0, len(self.auxButtonCmds)):
                if (self.auxButtonCmds[i] != ""):
                    auxButton = gtk.Button(self.auxButtonDescs[i])
                    auxButton.child.modify_font(pango.FontDescription("sans bold 30"))
                    auxButton.set_border_width(5)
                    auxButton.connect('clicked', self.auxWindowCallback, "auxbutton_%s" % i)
                    auxButton.modify_bg(gtk.STATE_SELECTED, gtk.gdk.color_parse("green"))

                    self.auxButtonWindowButtons.append(auxButton)

                    self.auxButtonWindowVBox.pack_start(auxButton, True, True, 0)

            self.auxButtonWindowCloseButton = gtk.Button(label='Close')
            self.auxButtonWindowCloseButton.child.modify_font(pango.FontDescription("sans bold 30"))
            self.auxButtonWindowCloseButton.set_border_width(5)
            self.auxButtonWindowCloseButton.connect('clicked', self.auxWindowCallback, 'close')
            self.auxButtonWindowCloseButton.modify_bg(gtk.STATE_SELECTED, gtk.gdk.color_parse("green"))

            self.auxButtonWindowButtons.append(self.auxButtonWindowCloseButton)

            (self.auxButtonWindowButtons[self.auxWindowButton]).set_state(gtk.STATE_SELECTED)

            self.auxButtonWindowVBox.pack_start(self.auxButtonWindowCloseButton, True, True, 0)

            self.auxButtonWindow.show_all()

            self.auxButtonWindowActive = True

    def powerWindowTimer(self, change = ''):
        if (self.powerButtonWindowActive):
            power = self.parentObject.signalTo('bluetooth', getPower = True)

            self.powerButtonWindowBluetoothButton.set_label(" turn %s\nBluetooth" % ("off" if (power == 1) else "on"))
            self.powerButtonWindowBluetoothButton.child.modify_font(pango.FontDescription("sans bold 20"))

            power = self.parentObject.signalTo('wifi', getPower = True)

            self.powerButtonWindowWifiButton.set_label("turn %s\n  WiFi" % ("off" if (power  == 1) else "on"))
            self.powerButtonWindowWifiButton.child.modify_font(pango.FontDescription("sans bold 20"))

            power = self.parentObject.signalTo('gps', getPower = True)

            self.powerButtonWindowGPSButton.set_label("turn %s\n  GPS" % ("off" if (power  == 1) else "on"))
            self.powerButtonWindowGPSButton.child.modify_font(pango.FontDescription("sans bold 20"))

        return self.powerButtonWindowActive

    def powerWindowCallback(self, widget, data = None, timeout = 3):
        if (data == 'togglebluetooth'):
            self.parentObject.signalTo('bluetooth', togglePower = True)

        elif (data == 'togglewifi'):
            self.parentObject.signalTo('wifi', togglePower = True)

        elif (data == 'togglegps'):
            self.parentObject.signalTo('gps', togglePower = True)

        elif (data == 'really_suspend'):
            self.powerButtonWindowDestroyer(None)

            time.sleep(0.5)

            self.pBWSuspBHandler = None

            self.parentObject.suspend()

            self.parentObject.resume()

            return False

        elif (data == 'suspend'):
            if (timeout >= 1):
                self.powerButtonWindowSuspendButton.set_label('  really\nSuspend')
                self.powerButtonWindowSuspendButton.child.modify_font(pango.FontDescription("sans bold 25"))

                if (self.pBWSuspBHandler):
                    self.powerButtonWindowSuspendButton.disconnect(self.pBWSuspBHandler)

                self.pBWSuspBHandler = self.powerButtonWindowSuspendButton.connect('clicked', self.powerWindowCallback, 'really_suspend')

                gobject.timeout_add_seconds(1, self.powerWindowCallback, None, 'suspend', timeout - 1)

                return False

            else:
                self.powerButtonWindowSuspendButton.set_label('\nSuspend\n')
                self.powerButtonWindowSuspendButton.child.modify_font(pango.FontDescription("sans bold 25"))
                if (self.pBWSuspBHandler):
                    self.powerButtonWindowSuspendButton.disconnect(self.pBWSuspBHandler)
                self.pBWSuspBHandler = self.powerButtonWindowSuspendButton.connect('clicked', self.powerWindowCallback, 'suspend')

                return False

        elif (data == 'really_lock'):
            self.powerButtonWindowDestroyer(None)

            self.pBWLockBHandler = None

            self.parentObject.signalTo('screenlock', lockScreen = True)

        elif (data == 'lock'):
            if (timeout >= 1):
                self.powerButtonWindowLockButton.set_label('    really\nLock Screen')
                self.powerButtonWindowLockButton.child.modify_font(pango.FontDescription("sans bold 25"))

                if (self.pBWLockBHandler):
                    self.powerButtonWindowLockButton.disconnect(self.pBWLockBHandler)

                self.pBWLockBHandler = self.powerButtonWindowLockButton.connect('clicked', self.powerWindowCallback, 'really_lock')

                gobject.timeout_add_seconds(1, self.powerWindowCallback, None, 'lock', timeout - 1)

                return False

            else:
                self.powerButtonWindowLockButton.set_label('\nLock Screen\n')
                self.powerButtonWindowLockButton.child.modify_font(pango.FontDescription("sans bold 25"))
                if (self.pBWLockBHandler):
                    self.powerButtonWindowLockButton.disconnect(self.pBWLockBHandler)
                self.pBWLockBHandler = self.powerButtonWindowLockButton.connect('clicked', self.powerWindowCallback, 'lock')

                return False

        elif (data == 'really_shutdown'):
            self.powerButtonWindowDestroyer(None)

            self.pBWShutBHandler = None

            if (self.shutdownExternal):
                os.system("%s &" % self.shutdownCommand)
            else:
                self.parentObject.shutdown()

        elif (data == 'shutdown'):
            if (self.confirmShutdown):
                if (timeout >= 1):
                    self.powerButtonWindowShutdownButton.set_label('   really\nShutdown')
                    self.powerButtonWindowShutdownButton.child.modify_font(pango.FontDescription("sans bold 25"))

                    if (self.pBWShutBHandler):
                        self.powerButtonWindowShutdownButton.disconnect(self.pBWShutBHandler)

                    self.pBWShutBHandler = self.powerButtonWindowShutdownButton.connect('clicked', self.powerWindowCallback, 'really_shutdown')

                    gobject.timeout_add_seconds(1, self.powerWindowCallback, None, 'shutdown', timeout - 1)

                    return False

                else:
                    self.powerButtonWindowShutdownButton.set_label('\nShutdown\n')
                    self.powerButtonWindowShutdownButton.child.modify_font(pango.FontDescription("sans bold 25"))
                    if (self.pBWShutBHandler):
                        self.powerButtonWindowShutdownButton.disconnect(self.pBWShutBHandler)
                    self.pBWShutBHandler = self.powerButtonWindowShutdownButton.connect('clicked', self.powerWindowCallback, 'shutdown')

                    return False

            else:
                self.powerWindowCallback(None, 'really_shutdown')

        elif (data == 'really_reboot'):
            self.powerButtonWindowDestroyer(None)

            self.pBWRebootBHandler = None

            if (self.rebootExternal):
                os.system("%s &" % self.rebootCommand)
            else:
               self.parentObject.reboot()

        elif (data == 'reboot'):
            if (self.confirmReboot):
                if (timeout >= 1):
                    self.powerButtonWindowRebootButton.set_label(' really\nReboot')
                    self.powerButtonWindowRebootButton.child.modify_font(pango.FontDescription("sans bold 25"))

                    if (self.pBWRebootBHandler):
                        self.powerButtonWindowRebootButton.disconnect(self.pBWRebootBHandler)

                    self.pBWRebootBHandler = self.powerButtonWindowRebootButton.connect('clicked', self.powerWindowCallback, 'really_reboot')

                    gobject.timeout_add_seconds(1, self.powerWindowCallback, None, 'reboot', timeout - 1)

                    return False

                else:
                    self.powerButtonWindowRebootButton.set_label('\nReboot\n')
                    self.powerButtonWindowRebootButton.child.modify_font(pango.FontDescription("sans bold 25"))
                    if (self.pBWRebootBHandler):
                        self.powerButtonWindowRebootButton.disconnect(self.pBWRebootBHandler)
                    self.pBWRebootBHandler = self.powerButtonWindowRebootButton.connect('clicked', self.powerWindowCallback, 'reboot')

                return False

            else:
                self.powerWindowCallback(None, 'really_reboot')

        
        elif (data == 'close'):
            self.powerButtonWindowDestroyer(None)

    def powerButtonWindowDestroyer(self, widget, data = None):
        self.powerButtonWindow.destroy()
        self.powerButtonWindowActive = False

    def powerButtonWindowDeleter(self, widget, event, data = None):
        return False

    def powerButtonWindowMaker(self):
        if (self.powerButtonWindowActive):
            self.powerButtonWindowDestroyer(None)
        else:
            self.powerWindowButton = 0

            self.powerButtonWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
            self.powerButtonWindow.set_title('Power')
            self.powerButtonWindow.set_border_width(2)
            self.powerButtonWindow.connect("delete_event", self.powerButtonWindowDeleter)
            self.powerButtonWindow.connect("destroy", self.powerButtonWindowDestroyer)
            self.powerButtonWindow.maximize()
            self.powerButtonWindow.show()

            self.powerButtonWindowFrame = gtk.Frame('Power (AUX cycles through buttons, POWER executes)')
            self.powerButtonWindow.add(self.powerButtonWindowFrame)

            self.powerButtonWindowVBox = gtk.VBox(False, 0)
            self.powerButtonWindowFrame.add(self.powerButtonWindowVBox)

            self.powerButtonWindowButtons = []

            self.powerButtonWindowHBox1 = None

            if (self.parentObject.iconEnabled('bluetooth')):
                self.powerButtonWindowHBox1 = gtk.HBox(True, 0)
                self.powerButtonWindowBluetoothButton = gtk.Button(" turn %s\nBluetooth" % ("off" if self.parentObject.signalTo('bluetooth', getPower = 1) else "on"))
                self.powerButtonWindowBluetoothButton.child.modify_font(pango.FontDescription("sans bold 20"))
                self.powerButtonWindowBluetoothButton.set_border_width(2)
                self.powerButtonWindowBluetoothButton.connect('clicked', self.powerWindowCallback, 'togglebluetooth')

                self.powerButtonWindowButtons.append(self.powerButtonWindowBluetoothButton)

                self.powerButtonWindowHBox1.pack_start(self.powerButtonWindowBluetoothButton)

            if (self.parentObject.iconEnabled('wifi')):
                if (not self.powerButtonWindowHBox1):
                    self.powerButtonWindowHBox1 = gtk.HBox(True, 0)

                self.powerButtonWindowWifiButton = gtk.Button("turn %s\n  WiFi" % ("off" if self.parentObject.signalTo('wifi', getPower = 1) else "on"))
                self.powerButtonWindowWifiButton.child.modify_font(pango.FontDescription("sans bold 20"))
                self.powerButtonWindowWifiButton.set_border_width(2)
                self.powerButtonWindowWifiButton.connect('clicked', self.powerWindowCallback, 'togglewifi')

                self.powerButtonWindowButtons.append(self.powerButtonWindowWifiButton)

                self.powerButtonWindowHBox1.pack_start(self.powerButtonWindowWifiButton)

            if (self.parentObject.iconEnabled('gps')):
                if (not self.powerButtonWindowHBox1):
                    self.powerButtonWindowHBox1 = gtk.HBox(True, 0)

                self.powerButtonWindowGPSButton = gtk.Button("turn %s\n  GPS" % ("off" if self.parentObject.signalTo('gps', getPower = 1) else "on"))
                self.powerButtonWindowGPSButton.child.modify_font(pango.FontDescription("sans bold 20"))
                self.powerButtonWindowGPSButton.set_border_width(2)
                self.powerButtonWindowGPSButton.connect('clicked', self.powerWindowCallback, 'togglegps')

                self.powerButtonWindowButtons.append(self.powerButtonWindowGPSButton)

                self.powerButtonWindowHBox1.pack_start(self.powerButtonWindowGPSButton)

            self.powerButtonWindowHBox2 = gtk.HBox(True, 0)

            self.powerButtonWindowShutdownButton = gtk.Button('\nShutdown\n')
            self.powerButtonWindowShutdownButton.child.modify_font(pango.FontDescription("sans bold 25"))
            self.powerButtonWindowShutdownButton.set_border_width(2)
            self.pBWShutBHandler = self.powerButtonWindowShutdownButton.connect('clicked', self.powerWindowCallback, 'shutdown')

            self.powerButtonWindowButtons.append(self.powerButtonWindowShutdownButton)

            self.powerButtonWindowHBox2.pack_start(self.powerButtonWindowShutdownButton)

            self.powerButtonWindowSuspendButton = gtk.Button('\nSuspend\n')
            self.powerButtonWindowSuspendButton.child.modify_font(pango.FontDescription("sans bold 25"))
            self.powerButtonWindowSuspendButton.set_border_width(2)
            self.pBWSuspBHandler = self.powerButtonWindowSuspendButton.connect('clicked', self.powerWindowCallback, 'suspend')

            self.powerButtonWindowButtons.append(self.powerButtonWindowSuspendButton)

            self.powerButtonWindowHBox2.pack_start(self.powerButtonWindowSuspendButton)

            self.powerButtonWindowHBox3 = gtk.HBox(True, 0)

            self.powerButtonWindowRebootButton = gtk.Button('\nReboot\n')
            self.powerButtonWindowRebootButton.child.modify_font(pango.FontDescription("sans bold 25"))
            self.powerButtonWindowRebootButton.set_border_width(2)
            self.pBWRebootBHandler = self.powerButtonWindowRebootButton.connect('clicked', self.powerWindowCallback, 'reboot')

            self.powerButtonWindowButtons.append(self.powerButtonWindowRebootButton)

            self.powerButtonWindowHBox3.pack_start(self.powerButtonWindowRebootButton)

            self.powerButtonWindowLockButton = gtk.Button('\nLock Screen\n')
            self.powerButtonWindowLockButton.child.modify_font(pango.FontDescription("sans bold 25"))
            self.powerButtonWindowLockButton.set_border_width(2)
            self.pBWLockBHandler = self.powerButtonWindowLockButton.connect('clicked', self.powerWindowCallback, 'lock')

            self.powerButtonWindowButtons.append(self.powerButtonWindowLockButton)

            self.powerButtonWindowHBox3.pack_start(self.powerButtonWindowLockButton)
 
            self.powerButtonWindowCloseButton = gtk.Button('Close')
#            self.powerButtonWindowCloseButton.child.child.get_children()[1].modify_font(pango.FontDescription("sans bold 25"))
            self.powerButtonWindowCloseButton.child.modify_font(pango.FontDescription("sans bold 40"))
            self.powerButtonWindowCloseButton.set_border_width(5)
            self.powerButtonWindowCloseButton.connect('clicked', self.powerWindowCallback, 'close')

            self.powerButtonWindowButtons.append(self.powerButtonWindowCloseButton)

            if (self.powerButtonWindowHBox1):
                self.powerButtonWindowVBox.pack_start(self.powerButtonWindowHBox1)

            self.powerButtonWindowVBox.pack_start(self.powerButtonWindowHBox2)
            self.powerButtonWindowVBox.pack_start(self.powerButtonWindowHBox3)
            self.powerButtonWindowVBox.pack_start(self.powerButtonWindowCloseButton, False)

            gobject.timeout_add_seconds(5, self.powerWindowTimer)

            self.powerButtonWindow.show_all()

            self.powerButtonWindowActive = True

    def configWindowHandler(self, widget, data = None):
        if (widget == self.auxHandleCheckButton):
            self.handleAux = self.auxHandleCheckButton.get_active()

        elif (widget == self.auxHandleShortCheckButton):
            self.handleAuxShort = self.auxHandleShortCheckButton.get_active()

        elif (widget == self.auxHandleShortExternalComboBox):
            self.handleAuxShortExternal = True if (self.auxHandleShortExternalComboBox.get_active() == 1) else False  

        elif (widget == self.auxHandleLongCheckButton):
            self.handleAuxLong = self.auxHandleLongCheckButton.get_active()

        elif (widget == self.auxHandleLongExternalComboBox):
            self.handleAuxLongExternal = True if (self.auxHandleLongExternalComboBox.get_active() == 1) else False

        elif (widget == self.auxHandleLongStartOnlyCheckButton):
            self.handleAuxLongOnlyStart = self.auxHandleLongStartOnlyCheckButton.get_active()

        # power
        elif (widget == self.powerHandleCheckButton):
            self.handlePower = self.powerHandleCheckButton.get_active()

        elif (widget == self.powerHandleShortCheckButton):
            self.handlePowerShort = self.powerHandleShortCheckButton.get_active()

        elif (widget == self.powerHandleShortExternalComboBox):
            self.handlePowerShortExternal = True if (self.powerHandleShortExternalComboBox.get_active() == 1) else False

        elif (widget == self.powerConfigShutdownExternalComboBox):
            self.shutdownExternal = True if (self.powerConfigShutdownExternalComboBox.get_active() == 1) else False

        elif (widget == self.powerConfigRebootExternalComboBox):
            self.rebootExternal = True if (self.powerConfigRebootExternalComboBox.get_active() == 1) else False

        elif (widget == self.powerHandleLongCheckButton):
            self.handlePowerLong = self.powerHandleLongCheckButton.get_active()

        elif (widget == self.powerHandleLongExternalComboBox):
            self.handlePowerLongExternal = True if (self.powerHandleLongExternalComboBox.get_active() == 1) else False

        elif (widget == self.powerHandleLongStartOnlyCheckButton):
            self.handlePowerLongOnlyStart = self.powerHandleLongStartOnlyCheckButton.get_active()

        # usb
        elif (widget == self.usbHandleCheckButton):
            self.handleUsb = self.usbHandleCheckButton.get_active()

        # headset
        elif (widget == self.headsetHandleCheckButton):
            self.handleHeadset = self.headsetHandleCheckButton.get_active()

        elif (widget == self.headsetHandleInsertionCheckButton):
            self.handleHeadsetInsertion = self.headsetHandleInsertionCheckButton.get_active()

        elif (widget == self.headsetHandleRemovalCheckButton):
            self.handleHeadsetRemoval = self.headsetHandleRemovalCheckButton.get_active()

        else:
            for i in range(0, 4):
                if (self.auxConfigButtonCmds[i].get_text() != ""):
                    self.auxButtonDescs[i] = self.auxConfigButtonDescs[i].get_text()
                    self.auxButtonCmds[i] = self.auxConfigButtonCmds[i].get_text()

            self.handleAuxShortExternalCommand = self.auxHandleShortExternalEntry.get_text()
            self.handleAuxLongExternalCommand = self.auxHandleLongExternalEntry.get_text()
            try:
                self.handleAuxLongTime = int(self.auxHandleLongExternalTimeEntry.get_text())
            except:
                self.handleAuxLongTime = 1

            self.handlePowerShortExternalCommand = self.powerHandleShortExternalEntry.get_text()
            self.handlePowerLongExternalCommand = self.powerHandleLongExternalEntry.get_text()
            try:
                self.handlePowerLongTime = int(self.powerHandleLongExternalTimeEntry.get_text())
            except:
                self.handlePowerLongTime = 1

            self.shutdownCommand = self.powerConfigShutdownCommandText.get_text()
            self.confirmShutdown = self.powerConfigConfirmShutdown.get_active()

            self.rebootCommand = self.powerConfigRebootCommandText.get_text()
            self.confirmReboot = self.powerConfigConfirmReboot.get_active()

            self.handleHeadsetInsertionCommand = self.headsetHandleInsertionCommandEntry.get_text()
            self.handleHeadsetRemovalCommand = self.headsetHandleRemovalCommandEntry.get_text()

        # aux
        self.auxHandleShortCheckButton.set_sensitive(self.handleAux)
        self.auxHandleShortExternalComboBox.set_sensitive(self.handleAux & self.handleAuxShort)
        self.auxHandleShortExternalEntry.set_sensitive(self.handleAux & self.handleAuxShort & self.handleAuxShortExternal)

        self.auxHandleLongCheckButton.set_sensitive(self.handleAux)
        self.auxHandleLongExternalComboBox.set_sensitive(self.handleAux & self.handleAuxLong)
        self.auxHandleLongExternalEntry.set_sensitive(self.handleAux & self.handleAuxLong & self.handleAuxLongExternal)
        self.auxHandleLongStartOnlyCheckButton.set_sensitive(self.handleAux & self.handleAuxLong & self.handleAuxLongExternal)

        self.auxHandleLongExternalTimeLabel1.set_sensitive(self.handleAux & self.handleAuxLong)
        self.auxHandleLongExternalTimeEntry.set_sensitive(self.handleAux & self.handleAuxLong)
        self.auxHandleLongExternalTimeLabel2.set_sensitive(self.handleAux & self.handleAuxLong)

        sensitive = self.handleAux & self.handleAuxLong & (not self.handleAuxLongExternal)
        for i in range(0, len(self.auxConfigButtonCmds)):
            self.auxConfigButtonCmds[i].set_sensitive(sensitive)
            self.auxConfigButtonDescs[i].set_sensitive(sensitive)
            self.auxConfigButtonLabels[i].set_sensitive(sensitive)

        # power
        self.powerHandleShortCheckButton.set_sensitive(self.handlePower)
        self.powerHandleShortExternalComboBox.set_sensitive(self.handlePower & self.handlePowerShort)
        self.powerHandleShortExternalEntry.set_sensitive(self.handlePower & self.handlePowerShort & self.handlePowerShortExternal)

        self.powerConfigShutdownCommandLabel.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))
        self.powerConfigShutdownExternalComboBox.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))
        self.powerConfigShutdownCommandText.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal) & self.shutdownExternal)
        self.powerConfigConfirmShutdown.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))

        self.powerConfigRebootCommandLabel.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))
        self.powerConfigRebootExternalComboBox.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))
        self.powerConfigRebootCommandText.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal) & self.rebootExternal)
        self.powerConfigConfirmReboot.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))

        self.powerHandleLongCheckButton.set_sensitive(self.handlePower)
        self.powerHandleLongExternalComboBox.set_sensitive(self.handlePower & self.handlePowerLong)
        self.powerHandleLongExternalEntry.set_sensitive(self.handlePower & self.handlePowerLong & self.handleAuxLongExternal)
        self.powerHandleLongStartOnlyCheckButton.set_sensitive(self.handlePower & self.handlePowerLong & self.handlePowerLongExternal)

        self.powerHandleLongExternalTimeLabel1.set_sensitive(self.handlePower & self.handlePowerLong)
        self.powerHandleLongExternalTimeEntry.set_sensitive(self.handlePower & self.handlePowerLong)
        self.powerHandleLongExternalTimeLabel2.set_sensitive(self.handlePower & self.handlePowerLong)

        #headset
        self.headsetHandleInsertionCheckButton.set_sensitive(self.handleHeadset)

        self.headsetHandleInsertionCommandLabel.set_sensitive(self.handleHeadset & self.handleHeadsetInsertion)
        self.headsetHandleInsertionCommandEntry.set_sensitive(self.handleHeadset & self.handleHeadsetInsertion)

        self.headsetHandleRemovalCheckButton.set_sensitive(self.handleHeadset)

        self.headsetHandleRemovalCommandLabel.set_sensitive(self.handleHeadset & self.handleHeadsetRemoval)
        self.headsetHandleRemovalCommandEntry.set_sensitive(self.handleHeadset & self.handleHeadsetRemoval)

    def getConfigFrame(self):
        self.buttonConfigNotebook = gtk.Notebook()
        self.buttonConfigNotebook.set_tab_pos(gtk.POS_TOP)
        self.buttonConfigNotebook.set_scrollable(True)

        self.auxScrolledWindow = gtk.ScrolledWindow()
        self.auxScrolledWindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.buttonConfigNotebook.append_page(self.auxScrolledWindow, gtk.Label("Aux"))

        self.auxVBox = gtk.VBox(False, 0)
        self.auxScrolledWindow.add_with_viewport(self.auxVBox)

        self.auxHandleCheckButton = gtk.CheckButton("Handle Aux-Button")
        self.auxHandleCheckButton.set_active(self.handleAux)
        self.auxHandleCheckButton.connect("clicked", self.configWindowHandler)

        # aux hort
        self.auxHBox1 = gtk.HBox(True, 0)
        
        self.auxHandleShortCheckButton = gtk.CheckButton("Handle short press")
        self.auxHandleShortCheckButton.set_active(self.handleAuxShort)
        self.auxHandleShortCheckButton.set_sensitive(self.handleAux)
        self.auxHandleShortCheckButton.connect("clicked", self.configWindowHandler)

        self.auxHandleShortExternalComboBox = gtk.combo_box_new_text()
        self.auxHandleShortExternalComboBox.append_text("internal")
        self.auxHandleShortExternalComboBox.append_text("external")
        self.auxHandleShortExternalComboBox.set_active(1 if self.handleAuxShortExternal else 0)
        self.auxHandleShortExternalComboBox.set_sensitive(self.handleAux & self.handleAuxShort)
        self.auxHandleShortExternalComboBox.connect("changed", self.configWindowHandler)

        self.auxHBox1.pack_start(self.auxHandleShortCheckButton)
        self.auxHBox1.pack_start(self.auxHandleShortExternalComboBox)

        self.auxHandleShortExternalEntry = gtk.Entry()
        self.auxHandleShortExternalEntry.set_text(self.handleAuxShortExternalCommand)
        self.auxHandleShortExternalEntry.set_sensitive(self.handleAux & self.handleAuxShort & self.handleAuxShortExternal)

        # aux long
        self.auxHBox2 = gtk.HBox(True, 0)
        
        self.auxHandleLongCheckButton = gtk.CheckButton("Handle long press")
        self.auxHandleLongCheckButton.set_active(self.handleAuxLong)
        self.auxHandleLongCheckButton.set_sensitive(self.handleAux)
        self.auxHandleLongCheckButton.connect("clicked", self.configWindowHandler)

        self.auxHandleLongExternalComboBox = gtk.combo_box_new_text()
        self.auxHandleLongExternalComboBox.append_text("internal")
        self.auxHandleLongExternalComboBox.append_text("external")
        self.auxHandleLongExternalComboBox.set_active(1 if self.handleAuxLongExternal else 0)
        self.auxHandleLongExternalComboBox.set_sensitive(self.handleAux & self.handleAuxLong)
        self.auxHandleLongExternalComboBox.connect("changed", self.configWindowHandler)

        self.auxHBox2.pack_start(self.auxHandleLongCheckButton)
        self.auxHBox2.pack_start(self.auxHandleLongExternalComboBox)

        self.auxHBox3 = gtk.HBox(False, 0)

        self.auxHandleLongExternalTimeLabel1 = gtk.Label("after")
        self.auxHandleLongExternalTimeLabel1.set_sensitive(self.handleAux & self.handleAuxLong & self.handleAuxLongExternal)
        
        self.auxHandleLongExternalTimeEntry = gtk.Entry()
        self.auxHandleLongExternalTimeEntry.set_text("%s" % self.handleAuxLongTime)
        self.auxHandleLongExternalTimeEntry.set_sensitive(self.handleAux & self.handleAuxLong & self.handleAuxLongExternal)

        self.auxHandleLongExternalTimeLabel2 = gtk.Label("seconds")
        self.auxHandleLongExternalTimeLabel2.set_sensitive(self.handleAux & self.handleAuxLong & self.handleAuxLongExternal)

        self.auxHBox3.pack_start(self.auxHandleLongExternalTimeLabel1)
        self.auxHBox3.pack_start(self.auxHandleLongExternalTimeEntry)
        self.auxHBox3.pack_start(self.auxHandleLongExternalTimeLabel2)

        self.auxHBox4 = gtk.HBox(False, 0)

        self.auxHandleLongExternalEntry = gtk.Entry()
        self.auxHandleLongExternalEntry.set_text(self.handleAuxLongExternalCommand) # anderer text wenn aus
        self.auxHandleLongExternalEntry.set_sensitive(self.handleAux & self.handleAuxLong & self.handleAuxLongExternal)

        self.auxHandleLongStartOnlyCheckButton = gtk.CheckButton("Start only")
        self.auxHandleLongStartOnlyCheckButton.set_active(self.handleAuxLongOnlyStart)
        self.auxHandleLongStartOnlyCheckButton.set_sensitive(self.handleAux & self.handleAuxLong & self.handleAuxLongExternal)
        self.auxHandleLongStartOnlyCheckButton.connect("clicked", self.configWindowHandler)
        
        self.auxHBox4.pack_start(self.auxHandleLongExternalEntry)
        self.auxHBox4.pack_start(self.auxHandleLongStartOnlyCheckButton, False, True)

        self.auxVBox.pack_start(self.auxHandleCheckButton, False)
        self.auxVBox.pack_start(gtk.HSeparator(), False)

        self.auxVBox.pack_start(self.auxHBox1, False)
        self.auxVBox.pack_start(self.auxHandleShortExternalEntry)
        self.auxVBox.pack_start(gtk.HSeparator(), False)

        self.auxVBox.pack_start(self.auxHBox2, False)
        self.auxVBox.pack_start(self.auxHBox3, False)
        self.auxVBox.pack_start(self.auxHBox4, False)

        self.auxConfigButtonLabels = [None, None, None, None]
        self.auxConfigButtonDescs = [None, None, None, None]
        self.auxConfigButtonCmds = [None, None, None, None]

        for i in range(0, 4):
            self.auxConfigButtonLabels[i] = gtk.Label("Aux Button %d" % (i+1))
            self.auxConfigButtonLabels[i].set_justify(gtk.JUSTIFY_LEFT)
            self.auxConfigButtonLabels[i].set_sensitive(not self.handleAuxLongExternal)

            auxButtonRow = gtk.HBox(True, 0)

            self.auxConfigButtonDescs[i] = gtk.Entry()
            self.auxConfigButtonDescs[i].set_text(self.auxButtonDescs[i])
            self.auxConfigButtonDescs[i].set_sensitive(not self.handleAuxLongExternal)

            self.auxConfigButtonCmds[i] = gtk.Entry()
            self.auxConfigButtonCmds[i].set_text(self.auxButtonCmds[i])
            self.auxConfigButtonCmds[i].set_sensitive(not self.handleAuxLongExternal)

            auxButtonRow.pack_start(self.auxConfigButtonDescs[i])
            auxButtonRow.pack_start(self.auxConfigButtonCmds[i])

            self.auxVBox.pack_start(self.auxConfigButtonLabels[i])
            self.auxVBox.pack_start(auxButtonRow, False)

        self.powerScrolledWindow = gtk.ScrolledWindow()
        self.powerScrolledWindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.buttonConfigNotebook.append_page(self.powerScrolledWindow, gtk.Label("Power"))

        self.powerVBox = gtk.VBox(False, 0)
        self.powerScrolledWindow.add_with_viewport(self.powerVBox)

        self.powerHandleCheckButton = gtk.CheckButton("Handle Power-Button")
        self.powerHandleCheckButton.set_active(self.handlePower)
        self.powerHandleCheckButton.connect("clicked", self.configWindowHandler)

        # power short
        self.powerHBox1 = gtk.HBox(True, 0)
        
        self.powerHandleShortCheckButton = gtk.CheckButton("Handle short press")
        self.powerHandleShortCheckButton.set_active(self.handlePowerShort)
        self.powerHandleShortCheckButton.set_sensitive(self.handlePower)
        self.powerHandleShortCheckButton.connect("clicked", self.configWindowHandler)

        self.powerHandleShortExternalComboBox = gtk.combo_box_new_text()
        self.powerHandleShortExternalComboBox.append_text("internal")
        self.powerHandleShortExternalComboBox.append_text("external")
        self.powerHandleShortExternalComboBox.set_active(1 if self.handlePowerShortExternal else 0)
        self.powerHandleShortExternalComboBox.set_sensitive(self.handlePower & self.handlePowerShort)
        self.powerHandleShortExternalComboBox.connect("changed", self.configWindowHandler)

        self.powerHBox1.pack_start(self.powerHandleShortCheckButton)
        self.powerHBox1.pack_start(self.powerHandleShortExternalComboBox)

        self.powerHandleShortExternalEntry = gtk.Entry()
        self.powerHandleShortExternalEntry.set_text(self.handlePowerShortExternalCommand)
        self.powerHandleShortExternalEntry.set_sensitive(self.handlePower & self.handlePowerShort & self.handlePowerShortExternal)

        self.powerHBox2 = gtk.HBox(True, 0)

        self.powerConfigShutdownCommandLabel = gtk.Label('Shutdown Command')
        self.powerConfigShutdownCommandLabel.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))

        self.powerConfigShutdownExternalComboBox = gtk.combo_box_new_text()
        self.powerConfigShutdownExternalComboBox.append_text("internal")
        self.powerConfigShutdownExternalComboBox.append_text("external")
        self.powerConfigShutdownExternalComboBox.set_active(1 if self.shutdownExternal else 0)
        self.powerConfigShutdownExternalComboBox.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))
        self.powerConfigShutdownExternalComboBox.connect("changed", self.configWindowHandler)

        self.powerHBox2.pack_start(self.powerConfigShutdownCommandLabel)
        self.powerHBox2.pack_start(self.powerConfigShutdownExternalComboBox)

        self.powerHBox21 = gtk.HBox(False, 0)

        self.powerConfigShutdownCommandText = gtk.Entry()
        self.powerConfigShutdownCommandText.set_text(self.shutdownCommand)
        self.powerConfigShutdownCommandText.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal) & self.shutdownExternal)

        self.powerConfigConfirmShutdown = gtk.CheckButton("need confirmation")
        self.powerConfigConfirmShutdown.set_active(self.confirmShutdown)
        self.powerConfigConfirmShutdown.connect("clicked", self.configWindowHandler)
        self.powerConfigConfirmShutdown.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))

        self.powerHBox21.pack_start(self.powerConfigShutdownCommandText)
        self.powerHBox21.pack_start(self.powerConfigConfirmShutdown, False, True)

        self.powerHBox3 = gtk.HBox(True, 0)

        self.powerConfigRebootCommandLabel = gtk.Label('Reboot Command')
        self.powerConfigRebootCommandLabel.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))

        self.powerConfigRebootExternalComboBox = gtk.combo_box_new_text()
        self.powerConfigRebootExternalComboBox.append_text("internal")
        self.powerConfigRebootExternalComboBox.append_text("external")
        self.powerConfigRebootExternalComboBox.set_active(1 if self.rebootExternal else 0)
        self.powerConfigRebootExternalComboBox.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))
        self.powerConfigRebootExternalComboBox.connect("changed", self.configWindowHandler)

        self.powerHBox3.pack_start(self.powerConfigRebootCommandLabel)
        self.powerHBox3.pack_start(self.powerConfigRebootExternalComboBox)

        self.powerHBox31 = gtk.HBox(False, 0)

        self.powerConfigRebootCommandText = gtk.Entry()
        self.powerConfigRebootCommandText.set_text(self.rebootCommand)
        self.powerConfigRebootCommandText.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal) & self.rebootExternal)

        self.powerConfigConfirmReboot = gtk.CheckButton("need confirmation")
        self.powerConfigConfirmReboot.set_active(self.confirmReboot)
        self.powerConfigConfirmReboot.connect("clicked", self.configWindowHandler)
        self.powerConfigConfirmReboot.set_sensitive(self.handlePower & self.handlePowerShort & (not self.handlePowerShortExternal))

        self.powerHBox31.pack_start(self.powerConfigRebootCommandText)
        self.powerHBox31.pack_start(self.powerConfigConfirmReboot, False, True)

        # power long
        self.powerHBox4 = gtk.HBox(True, 0)
        
        self.powerHandleLongCheckButton = gtk.CheckButton("Handle long press")
        self.powerHandleLongCheckButton.set_active(self.handlePowerLong)
        self.powerHandleLongCheckButton.set_sensitive(self.handlePower)
        self.powerHandleLongCheckButton.connect("clicked", self.configWindowHandler)

        self.powerHandleLongExternalComboBox = gtk.combo_box_new_text()
        self.powerHandleLongExternalComboBox.append_text("internal")
        self.powerHandleLongExternalComboBox.append_text("external")
        self.powerHandleLongExternalComboBox.set_active(1 if self.handlePowerLongExternal else 0)
        self.powerHandleLongExternalComboBox.set_sensitive(self.handlePower & self.handlePowerLongExternal)
        self.powerHandleLongExternalComboBox.connect("changed", self.configWindowHandler)

        self.powerHBox4.pack_start(self.powerHandleLongCheckButton)
        self.powerHBox4.pack_start(self.powerHandleLongExternalComboBox)

        self.powerHBox5 = gtk.HBox(False, 0)

        self.powerHandleLongExternalTimeLabel1 = gtk.Label("after")
        self.powerHandleLongExternalTimeLabel1.set_sensitive(self.handlePower & self.handlePowerLong & self.handlePowerLongExternal)
        
        self.powerHandleLongExternalTimeEntry = gtk.Entry()
        self.powerHandleLongExternalTimeEntry.set_text("%s" % self.handlePowerLongTime)
        self.powerHandleLongExternalTimeEntry.set_sensitive(self.handlePower & self.handlePowerLong & self.handlePowerLongExternal)

        self.powerHandleLongExternalTimeLabel2 = gtk.Label("seconds")
        self.powerHandleLongExternalTimeLabel2.set_sensitive(self.handleAux & self.handlePowerLong & self.handleAuxLongExternal)

        self.powerHBox5.pack_start(self.powerHandleLongExternalTimeLabel1)
        self.powerHBox5.pack_start(self.powerHandleLongExternalTimeEntry)
        self.powerHBox5.pack_start(self.powerHandleLongExternalTimeLabel2)

        self.powerHBox6 = gtk.HBox(False, 0)

        self.powerHandleLongExternalEntry = gtk.Entry()
        self.powerHandleLongExternalEntry.set_text(self.handlePowerLongExternalCommand)
        self.powerHandleLongExternalEntry.set_sensitive(self.handlePower & self.handlePowerLong & self.handlePowerLongExternal)

        self.powerHandleLongStartOnlyCheckButton = gtk.CheckButton("Start only")
        self.powerHandleLongStartOnlyCheckButton.set_active(self.handlePowerLongOnlyStart)
        self.powerHandleLongStartOnlyCheckButton.set_sensitive(self.handlePower & self.handlePowerLong & self.handlePowerLongExternal)
        self.powerHandleLongStartOnlyCheckButton.connect("clicked", self.configWindowHandler)
        
        self.powerHBox6.pack_start(self.powerHandleLongExternalEntry)
        self.powerHBox6.pack_start(self.powerHandleLongStartOnlyCheckButton, False, True)

        self.powerVBox.pack_start(self.powerHandleCheckButton, False)
        self.powerVBox.pack_start(gtk.HSeparator(), False)

        self.powerVBox.pack_start(self.powerHBox1, False)
        self.powerVBox.pack_start(self.powerHandleShortExternalEntry, False)
        self.powerVBox.pack_start(gtk.HSeparator(), False)

        self.powerVBox.pack_start(self.powerHBox2, False)
        self.powerVBox.pack_start(self.powerHBox21, False)
        self.powerVBox.pack_start(gtk.HSeparator(), False)

        self.powerVBox.pack_start(self.powerHBox3, False)
        self.powerVBox.pack_start(self.powerHBox31, False)
        self.powerVBox.pack_start(gtk.HSeparator(), False)

        self.powerVBox.pack_start(self.powerHBox4, False)
        self.powerVBox.pack_start(self.powerHBox5, False)
        self.powerVBox.pack_start(self.powerHBox6, False)

        # usb
        self.usbScrolledWindow = gtk.ScrolledWindow()
        self.usbScrolledWindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.buttonConfigNotebook.append_page(self.usbScrolledWindow, gtk.Label("Usb"))

        self.usbVBox = gtk.VBox(False, 0)
        self.usbScrolledWindow.add_with_viewport(self.usbVBox)

        self.usbHandleCheckButton = gtk.CheckButton("Handle usb")
        self.usbHandleCheckButton.set_active(self.handleUsb)
        self.usbHandleCheckButton.connect("clicked", self.configWindowHandler)

        self.usbVBox.pack_start(self.usbHandleCheckButton, False)

        # headset
        self.headsetScrolledWindow = gtk.ScrolledWindow()
        self.headsetScrolledWindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.buttonConfigNotebook.append_page(self.headsetScrolledWindow, gtk.Label("Headset"))

        self.headsetVBox = gtk.VBox(False, 0)
        self.headsetScrolledWindow.add_with_viewport(self.headsetVBox)

        self.headsetHandleCheckButton = gtk.CheckButton("Handle Headset")
        self.headsetHandleCheckButton.set_active(self.handleHeadset)
        self.headsetHandleCheckButton.connect("clicked", self.configWindowHandler)

        self.headsetHandleInsertionCheckButton = gtk.CheckButton("Handle Insertion")
        self.headsetHandleInsertionCheckButton.set_active(self.handleHeadsetInsertion)
        self.headsetHandleInsertionCheckButton.set_sensitive(self.handleHeadset)
        self.headsetHandleInsertionCheckButton.connect("clicked", self.configWindowHandler)

        self.headsetHandleInsertionCommandLabel = gtk.Label("Insertion Command")
        self.headsetHandleInsertionCommandLabel.set_sensitive(self.handleHeadset & self.handleHeadsetInsertion)

        self.headsetHandleInsertionCommandEntry = gtk.Entry()
        self.headsetHandleInsertionCommandEntry.set_text(self.handleHeadsetInsertionCommand)
        self.headsetHandleInsertionCommandEntry.set_sensitive(self.handleHeadset & self.handleHeadsetInsertion)

        self.headsetHandleRemovalCheckButton = gtk.CheckButton("Handle Removal")
        self.headsetHandleRemovalCheckButton.set_active(self.handleHeadsetRemoval)
        self.headsetHandleRemovalCheckButton.set_sensitive(self.handleHeadset)
        self.headsetHandleRemovalCheckButton.connect("clicked", self.configWindowHandler)

        self.headsetHandleRemovalCommandLabel = gtk.Label("Removal Command")
        self.headsetHandleRemovalCommandLabel.set_sensitive(self.handleHeadset & self.handleHeadsetRemoval)

        self.headsetHandleRemovalCommandEntry = gtk.Entry()
        self.headsetHandleRemovalCommandEntry.set_text(self.handleHeadsetRemovalCommand)
        self.headsetHandleRemovalCommandEntry.set_sensitive(self.handleHeadset & self.handleHeadsetRemoval)

        self.headsetVBox.pack_start(self.headsetHandleCheckButton, False)

        self.headsetVBox.pack_start(self.headsetHandleInsertionCheckButton, False)
        self.headsetVBox.pack_start(self.headsetHandleInsertionCommandLabel, False)
        self.headsetVBox.pack_start(self.headsetHandleInsertionCommandEntry, False)

        self.headsetVBox.pack_start(self.headsetHandleRemovalCheckButton, False)
        self.headsetVBox.pack_start(self.headsetHandleRemovalCommandLabel, False)
        self.headsetVBox.pack_start(self.headsetHandleRemovalCommandEntry, False)

        return self.buttonConfigNotebook
