# vim: set fileencoding=utf-8 :

import gobject
import gtk
import logging

from panelplugin.UsageIcon import UsageIcon

#----------------------------------------------------------------------------#
class GSMIcon(UsageIcon):
#----------------------------------------------------------------------------#
    def __init__(self, parentObject = None, notificationAvailable = False):
        self.parentObject = parentObject
        self.notificationAvailable = notificationAvailable

        self.gsmStatus = None
        self.gsmStatusChanged = False
        self.gsmStrength = None
        self.gsmRegistration = None
        self.gsmAct = None
        self.gsmProvider = None
        self.gsmLac = None
        self.gsmCid = None

        self.pdpApn = ''
        self.pdpUsername = ''
        self.pdpPassword = ''
        self.pdpStatus = None
        self.pdpRegistration = None
        self.pdpAct = None
        self.pdpLac = None
        self.pdpCid = None
        
        self.pdpContextId = None
        self.pdpContextStatus = None
        self.pdpContextProperties = None

        UsageIcon.__init__(self,
                           "Turn on GSM",
                           "Turn off GSM",
                           "GSM",
                           self.parentObject,
                           "gsm",
                           self.notificationAvailable)

        self.menuItemGPRSOn = None
        self.menuItemGPRSOff = None

        self.dbusGSMStatusSignal = None
        self.dbusGSMSignalStrengthSignal = None

        self.dbusPDPNetworkStatusSignal = None
        self.dbusPDPContextStatusSignal = None

        self.dbusCallStatusSignal = None

        self.gsmNetworkIface = None
        self.gsmPDPIface = None
        self.gsmCallIface = None

        self.dbusReInit()

    def dbusReInit(self, failure = False):
        self.gsmNetworkIface = None
        self.gsmPDPIface = None
        self.gsmCallIface = None

        UsageIcon.dbusReInit(self, failure)

    def deferredDBusInit(self):
        if (UsageIcon.deferredDBusInit(self)):
            return True

        logging.info('entering init for gsm')

        self.gsmNetworkIface = self.parentObject.tryGetInterface('org.freesmartphone.ogsmd',
                                                                 '/org/freesmartphone/GSM/Device',
                                                                 'org.freesmartphone.GSM.Network')


        self.gsmPDPIface = self.parentObject.tryGetInterface('org.freesmartphone.ogsmd',
                                                             '/org/freesmartphone/GSM/Device',
                                                             'org.freesmartphone.GSM.PDP')

        self.gsmCallIface = self.parentObject.tryGetInterface('org.freesmartphone.ogsmd',
                                                              '/org/freesmartphone/GSM/Device',
                                                              'org.freesmartphone.GSM.Call')

        if ((self.gsmNetworkIface) and (self.gsmPDPIface) and (self.gsmCallIface)):
            if (self.dbusGSMStatusSignal):
                try:
                    self.dbusGSMStatusSignal.remove()
                    self.dbusGSMStatusSignal = None
                except Exception, e:
                    logging.error("error while disconnecting from signal 'GSM.Status' \n\t(%s)" % e)

            if (not self.dbusGSMStatusSignal):
                self.dbusGSMStatusSignal = self.gsmNetworkIface.connect_to_signal("Status", self.updateGSMNetworkStatus)

            if (self.dbusGSMSignalStrengthSignal):
                try:
                    self.dbusGSMSignalStrengthSignal.remove()
                    self.dbusGSMSignalStrengthSignal = None
                except Exception, e:
                    logging.error("error while disconnecting from signal 'GSM.SignalStrength' \n\t(%s)" % e)

            if (not self.dbusGSMSignalStrengthSignal):
                self.dbusGSMSignalStrengthSignal = self.gsmNetworkIface.connect_to_signal("SignalStrength", self.updateGSMSignalStrength)

            if (self.dbusPDPNetworkStatusSignal):
                try:
                    self.dbusPDPNetworkStatusSignal.remove()
                    self.dbusPDPNetworkStatusSignal = None
                except Exception, e:
                    logging.error("error while disconnecting from signal 'PDP.NetworkStatus' \n\t(%s)" % e)

            if (not self.dbusPDPNetworkStatusSignal):
                self.dbusPDPNetworkStatusSignal = self.gsmPDPIface.connect_to_signal("NetworkStatus", self.updatePDPNetworkStatus)

            if (self.dbusPDPContextStatusSignal):
                try:
                    self.dbusPDPContextStatusSignal.remove()
                    self.dbusPDPContextStatusSignal = None
                except Exception, e:
                    logging.error("error while disconnecting from signal 'PDP.ContextStatus' \n\t(%s)" % e)

            if (not self.dbusPDPContextStatusSignal):
                self.dbusPDPContextStatusSignal = self.gsmPDPIface.connect_to_signal("ContextStatus", self.updatePDPContextStatus)

            if (not self.menuItemGPRSOn):
                self.menuItemGPRSOn = gtk.MenuItem("Turn on GPRS")
                self.menuItemGPRSOn.connect("activate", self.activateMenu)
                self.menuItemGPRSOn.show()
                self.menuItemGPRSOn.set_sensitive(False)
                self.menu.append(self.menuItemGPRSOn)

            if (not self.menuItemGPRSOff):
                self.menuItemGPRSOff = gtk.MenuItem("Turn off GPRS")
                self.menuItemGPRSOff.connect("activate", self.activateMenu)
                self.menuItemGPRSOff.show()
                self.menuItemGPRSOff.set_sensitive(False)
                self.menu.append(self.menuItemGPRSOff)

            if (self.dbusCallStatusSignal):
                try:
                    self.dbusCallStatusSignal.remove()
                    self.dbusCallStatusSignal = None
                except Exception, e:
                    logging.error("error while disconnecting from signal 'CallStatus' \n\t(%s)" % e)

            if (not self.dbusCallStatusSignal):
                self.dbusCallStatusSignal = self.gsmCallIface.connect_to_signal("CallStatus", self.updateCallStatus)

            self.update()

            self.startupDone = True

            return False

        logging.warning('entering deferred init for gsm')

        return True

    def configReader(self):
        if (UsageIcon.configReader(self)):
            if (self.config.has_option(self.configName, 'pdpapn')):
                self.pdpApn = self.config.get(self.configName, 'pdpapn')

            if (self.config.has_option(self.configName, 'pdpusername')):
                self.pdpUsername = self.config.get(self.configName, 'pdpusername')

            if (self.config.has_option(self.configName, 'pdppassword')):
                self.pdpPassword = self.config.get(self.configName, 'pdppassword')

            return True

        return False

    def configWriter(self):
        UsageIcon.configWriter(self)

        if (self.pdpApn == ''):
            try:
                self.config.remove_option(self.configName, 'pdpapn')
            except:
                pass
        else:
            self.config.set(self.configName, 'pdpapn', self.pdpApn)

        if (self.pdpUsername == ''):
            try:
                self.config.remove_option(self.configName, 'pdpusername')
            except:
                pass
        else:
            self.config.set(self.configName, 'pdpusername', self.pdpUsername)

        if (self.pdpPassword == ''):
            try:
                self.config.remove_option(self.configName, 'pdppassword')
            except:
                pass
        else:
            self.config.set(self.configName, 'pdppassword', self.pdpPassword)

    def activateMenu(self, widget, data = None):
        if (widget == self.menuItemGPRSOn):
            self.pdpActivateContext()

        elif (widget == self.menuItemGPRSOff):
            self.pdpDeactivateContext()

        UsageIcon.activateMenu(self, widget, data)

    def signal(self, **kwargs):
        if ('gprsOn' in kwargs):
            if (self.resourceState):
                self.pdpActivateContext()

        elif ('gprsOff' in kwargs):
            self.pdpDeactivateContext()

        else:
            return UsageIcon.signal(self, **kwargs)

    def resourceChanged(self, name, state, attributes):
        if (state == 0):
            self.gsmAct = None
            self.gsmLac = None
            self.gsmProvider = None
            self.gsmRegistration = None
            self.gsmStatus = None
            self.gsmStrength = None

        UsageIcon.resourceChanged(self, name, state, attributes)

    def updateCallStatus(self, id, status, properties):
        logging.info("callStatus: %s" % status)

        self.parentObject.signalTo('screenlock', callStatus = status)

    def updateGSMSignalStrength(self, signalstrength = 0):
        if (self.gsmStrength != signalstrength):
            self.gsmStrength = signalstrength
            
            self.updateIcon()

    def updateGSMNetworkStatus(self, status = {}):
        logging.info("gsm status: %s" % status)

        self.gsmStatus = status
        sendNotification = False

        if ('registration' in self.gsmStatus):
            registration = self.gsmStatus[u'registration']
            if (self.gsmRegistration != registration):
                self.gsmRegistration = registration
                sendNotification = True
        else:
            self.gsmRegistration = None

        if ('act' in self.gsmStatus):
            act = self.gsmStatus[u'act']
            if (self.gsmAct != act):
                self.gsmAct = act
        else:
            self.gsmAct = None

        if ('strength' in self.gsmStatus):
            self.gsmStrength = self.gsmStatus[u'strength']
        else:
            self.gsmStrength = None

        if ('provider' in self.gsmStatus):
            provider = self.gsmStatus[u'provider']
            if (self.gsmProvider != provider):
                self.gsmProvider = provider
                sendNotification = True
        else:
            self.gsmProvider = None

        if ('lac' in self.gsmStatus):
            if (self.gsmLac != self.gsmStatus[u'lac']):
                self.gsmLac = self.gsmStatus[u'lac']
                sendNotification = True

        if ('cid' in self.gsmStatus):
            if (self.gsmCid != self.gsmStatus[u'cid']):
                self.gsmCid = self.gsmStatus[u'cid']
                sendNotification = True

        if (sendNotification):
            self.gsmStatusChanged = True
            if (self.notificationAvailable):
                self.commitNotification()

        self.updateIcon()

    def updatePDPNetworkStatus(self, status = {}):
        logging.info("pdp status: %s" % status)
        
        self.pdpStatus = status
        sendNotification = False

        if ('registration' in self.pdpStatus):
            registration = self.pdpStatus[u'registration']
            if (self.pdpRegistration != registration):
                self.pdpRegistration = registration
        else:
            self.pdpRegistration = None

        if ('act' in self.pdpStatus):
            act = self.pdpStatus[u'act']
            if (self.pdpAct != act):
                self.pdpAct = act
        else:
            self.pdpAct = None

        if ('lac' in self.pdpStatus):
            lac = self.pdpStatus[u'lac']
            if (self.pdpLac != lac):
                self.pdpLac = lac

        if ('cid' in self.pdpStatus):
            cid = self.pdpStatus[u'cid']
            if (self.pdpCid != cid):
                self.pdpCid = cid

        self.updateIcon()

    def updatePDPContextStatus(self, id, status, properties = None):
        logging.info("id: %s status: %s properties: %s" % (id, status, properties))

        sendNotification = False

        if (self.pdpContextId != id):
            self.pdpContextId = id

        if (self.pdpContextStatus != status):
            self.pdpContextStatus = status
            sendNotification = True

        if (self.pdpContextProperties != properties):
            self.pdpContextProperties = properties

        if (sendNotification):
            self.gsmStatusChanged = True
            if (self.notificationAvailable):
                self.commitNotification()

        self.updateIcon()

    def pdpActivateContext(self):
        logging.info("ActivateContext")

        try:
            self.gsmPDPIface.ActivateContext(self.pdpApn, self.pdpUsername, self.pdpPassword)
        except Exception, e:
            logging.warning("error while activating pdpContext\n\t(%s)" % e)

    def pdpDeactivateContext(self):
        logging.info("DeactivateContext")

        try:
            self.gsmPDPIface.DeactivateContext()
        except Exception, e:
            logging.warning("error while activating pdpContext\n\t(%s)" % e)

    def postEnable(self):
        if (self.menuItemGPRSOn):
            self.menuItemGPRSOn.set_sensitive(True)

        if (self.menuItemGPRSOff):
            self.menuItemGPRSOff.set_sensitive(True)
        
        UsageIcon.postEnable(self)

    def postDisable(self):
        if (self.menuItemGPRSOn):
            self.menuItemGPRSOn.set_sensitive(False)

        if (self.menuItemGPRSOff):
            self.menuItemGPRSOff.set_sensitive(False)

        UsageIcon.postDisable(self)

    def getIcon(self):
        if ((self.gsmStrength) and (self.resourceState != 0)):
            if ((self.gsmStrength >= 0) and (self.gsmStrength < 20)):
                icon = "%s/gsm_000.png" % self.parentObject.iconpath
            elif ((self.gsmStrength >= 20) and (self.gsmStrength < 40)):
                icon = "%s/gsm_020.png" % self.parentObject.iconpath
            elif ((self.gsmStrength >= 40) and (self.gsmStrength < 60)):
                icon =  "%s/gsm_040.png" % self.parentObject.iconpath
            elif ((self.gsmStrength >= 60) and (self.gsmStrength < 80)):
                icon =  "%s/gsm_060.png" % self.parentObject.iconpath
            elif (self.gsmStrength >= 80):
                icon = "%s/gsm_100.png" % self.parentObject.iconpath
        else:
            if (self.resourceState == 0):
                icon = "%s/gsm_0.png" % self.parentObject.iconpath
            else:
                icon = "%s/gsm_000.png" % self.parentObject.iconpath

        return icon
    
    def getTooltip(self):
        tooltip = UsageIcon.getTooltip(self)

        if (self.gsmRegistration):
            if (self.gsmRegistration != "unregistered"):
                if (self.gsmRegistration == "unknown"):
                    tooltip += "\nregistration unknown"
                elif (self.gsmRegistration == "denied"):
                    tooltip += "\nno permitted network available"
                elif (self.gsmRegistration == "busy"):
                    tooltip += "\nnot registered, but currently trying"
                elif (self.gsmRegistration in ["home", "roaming"]):
                    if (self.gsmRegistration == "home"):
                        tooltip += "\nHome "
                    elif (self.gsmRegistration == "roaming"):
                        tooltip += "\nRoaming "

                    if (self.gsmProvider):
                        tooltip += self.gsmProvider

                    if (self.gsmAct):
                        tooltip += " (%s)" % self.gsmAct

                    if ((self.gsmLac) and (self.gsmCid)):
                        tooltip += "\nin Cell %s:%s" % (self.gsmLac, self.gsmCid)

                    if (self.gsmStrength):
                        tooltip += " at %s%%" % self.gsmStrength

        if (self.pdpRegistration):
            if (self.pdpRegistration != "unregistered"):
                tooltip += "\nGPRS "
                
                if (self.pdpContextStatus):
                    if (self.pdpContextStatus == "incoming"):
                        tooltip += "incoming\n (but not yet accepted)"
                    elif (self.pdpContextStatus == "outgoing"):
                        tooltip += "outgoing\n (but not yet established)"
                    elif (self.pdpContextStatus == "active"):
                        tooltip += "active\n (have a new default route)"
                    elif (self.pdpContextStatus == "held"):
                        tooltip += "held\n (interrupted by a voice call)"
                    elif (self.pdpContextStatus == "release"):
                        tooltip += "inactive"

        return tooltip

    def getNotificationText(self):
        notification = ""

        if (self.gsmStatusChanged):
            notification = self.getTooltip()
            self.gsmStatusChanged = False
        else:
            notification = UsageIcon.getNotificationText(self)

        return notification

    def update(self, execute = False):
        if (execute):
            if ((self.resourceState == 1) & (self.gsmNetworkIface != None)):
                try:
                    status = self.gsmNetworkIface.GetStatus()
                    self.updateGSMNetworkStatus(status)

                except Exception, e:
                    logging.error('error getting gsm network status\n\t(%s)' % e)
                    self.dbusReInit(True)

#                if (self.pdpContextId):
                if (True & (self.gsmPDPIface != None)):
                    try:
#                        status = self.gsmPDPIface.GetContextStatus(self.pdpContextId)
                        status = self.gsmPDPIface.GetContextStatus()
                        self.updatePDPContextStatus(self.pdpContextId, status)

                    except Exception, e:
                        logging.error('error getting pdp context status\n\t(%s)' % e)

                try:
                    status = self.gsmPDPIface.GetNetworkStatus()
                    self.updatePDPNetworkStatus(status)

                except Exception, e:
                    logging.error('error getting pdp network status\n\t(%s)' % e)

            self.updateIcon()
        else:
            gobject.timeout_add(1, self.update, True)

        return ((not self.parentObject.thread_terminate) and (not execute))

    def configWindowHandler(self, widget, data = None):
        self.pdpApn = self.pdpApnEntry.get_text()
        self.pdpUsername = self.pdpUsernameEntry.get_text()
        self.pdpPassword = self.pdpPasswordEntry.get_text()

        UsageIcon.configWindowHandler(self, widget, data)

    def getConfigFrame(self):
        configFrame = UsageIcon.getConfigFrame(self)

        self.pdpHBox = gtk.HBox(False, 0)

        self.pdpLabel = gtk.Label('GPRS-Settings')

        self.pdpVBox1 = gtk.VBox(True, 0)

        self.pdpApnLabel = gtk.Label('APN:')
        self.pdpUsernameLabel = gtk.Label('Username:')
        self.pdpPasswordLabel = gtk.Label('Password:')

        self.pdpVBox1.pack_start(self.pdpApnLabel)
        self.pdpVBox1.pack_start(self.pdpUsernameLabel)
        self.pdpVBox1.pack_start(self.pdpPasswordLabel)

        self.pdpVBox2 = gtk.VBox(True, 0)

        self.pdpApnEntry = gtk.Entry()
        self.pdpApnEntry.set_text(self.pdpApn)

        self.pdpUsernameEntry = gtk.Entry()
        self.pdpUsernameEntry.set_text(self.pdpUsername)

        self.pdpPasswordEntry = gtk.Entry()
        self.pdpPasswordEntry.set_text(self.pdpPassword)

        self.pdpVBox2.pack_start(self.pdpApnEntry)
        self.pdpVBox2.pack_start(self.pdpUsernameEntry)
        self.pdpVBox2.pack_start(self.pdpPasswordEntry)        

        self.pdpHBox.pack_start(self.pdpVBox1)
        self.pdpHBox.pack_start(self.pdpVBox2)

        self.configFrameVBox.pack_start(self.pdpLabel)
        self.configFrameVBox.pack_start(self.pdpHBox)

        return configFrame
