# vim: set fileencoding=utf-8 :

import os
import gtk
import signal
import subprocess
from panelplugin.StatusIcon import StatusIcon

#----------------------------------------------------------------------------#
class KeyboardIcon(StatusIcon):
#----------------------------------------------------------------------------#
    def __init__(self, parentObject = None, notificationAvailable = False):
        self.parentObject = parentObject
        self.notificationAvailable = notificationAvailable

        self.keyboardBin = "/usr/bin/matchbox-keyboard"
        self.keyboardPid = 0

        StatusIcon.__init__(self, self.parentObject, "keyboard")
        self.dbusReInit()

    def configReader(self):
        StatusIcon.configReader(self)

        if (self.config.has_section(self.configName)):
            if (self.config.has_option(self.configName, "keyboard")):
                self.keyboardBin = self.config.get(self.configName, "keyboard")

    def configWriter(self):
        StatusIcon.configWriter(self)

        self.config.set(self.configName, "keyboard", self.keyboardBin)

    def getIcon(self):
        return "%s/matchbox-keyboard_%d.png" % (self.parentObject.iconpath, self.getKdbStatus())

    def getTooltip(self):
        return "Onscreen Keyboard"

    def getKdbStatus(self):
        if (self.keyboardPid != 0):
            return True
        else:
            return False

    def activate(self, widget, data = None):
        if (self.getKdbStatus()):
            os.kill(self.keyboardPid, signal.SIGTERM)
            os.waitpid(self.keyboardPid, 0)
            self.keyboardPid = 0
        else:
            self.keyboardPid = subprocess.Popen(self.keyboardBin, shell = False, close_fds = True).pid

        self.updateIcon()       

    def update(self):
        self.updateIcon()       

    def configWindowHandler(self, widget, data = None):
        if (self.configCheckButtonVisible.get_active()):
            self.setVisible(True)
        else:
            self.setVisible(False)

        self.keyboardBin = self.configKeyboardCmdText.get_text()

    def getConfigFrame(self):
        self.configFrame = gtk.Frame()

        self.configScrolledWindow = gtk.ScrolledWindow()
        self.configScrolledWindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.configFrame.add(self.configScrolledWindow)

        self.configFrameVBox = gtk.VBox(False, 0)
        self.configScrolledWindow.add_with_viewport(self.configFrameVBox)

        self.configCheckButtonVisible = gtk.CheckButton("Visible")
        self.configCheckButtonVisible.set_active(self.visible)
        self.configCheckButtonVisible.connect("clicked", self.configWindowHandler)

        self.configKeyboardCmdLabel = gtk.Label('Keyboard Command')
        self.configKeyboardCmdLabel.set_justify(gtk.JUSTIFY_LEFT)

        self.configKeyboardCmdText = gtk.Entry()
        self.configKeyboardCmdText.set_text(self.keyboardBin)

        self.configFrameVBox.pack_start(self.configCheckButtonVisible)
        self.configFrameVBox.pack_start(self.configKeyboardCmdLabel)
        self.configFrameVBox.pack_start(self.configKeyboardCmdText)

        return self.configFrame
