# vim: set fileencoding=utf-8 :

import ConfigParser # for config-files
import gobject
import gtk
import logging
import os
import pango # for font-size
import time

from dbus import Interface
from dbus.exceptions import DBusException

#----------------------------------------------------------------------------#
class PanelPlugin():
#----------------------------------------------------------------------------#
    def __init__(self, dbus, notificationAvailable = False):
        self.dbus = dbus
        self.notificationAvailable = notificationAvailable

        self.starting = True
        self.iconpath = '/usr/share/pixmaps/openmoko-panel-plugin'
        self.availableIcons = ['buttons', 'usb', 'gsm', 'wifi', 'gps', 'keyboard', 'brightness', 'battery', 'bluetooth', 'screenlock']
        self.enabledIcons = self.availableIcons
        self.iconOrder = []
        self.icons = []
        self.screenlockicon = None
        self.configWindowActiveIconsCheckBoxes = {}
        self.runningUpdate = False

        self.thread_terminate = False

        try:
            self.kernel = os.popen('uname -r', 'r').readline().rstrip('\n')[0:6]
        except Exception, e:
            logging.info('assuming kernel is 2.6.24\n\t(%s)' % e)
            self.kernel = '2.6.24'

        self.configWindowActive = False
        self.configFiles = ['/etc/panel-plugin.conf', os.path.expanduser('~/.panel-pluginrc')]
        self.panelConfig = ConfigParser.SafeConfigParser()

        self.configReader()

        self.starting = False

        self.dbusReInit()

    def spreadDBusFailure(self):
        logging.info('entering spreadDBusFailure')

        for icon in self.icons:
            gobject.timeout_add_seconds(10, icon.deferredDBusInit)

        gobject.timeout_add_seconds(10, self.deferredDBusInit)

    def dbusReInit(self):
        self.usageIface = None

        gobject.timeout_add_seconds(10, self.deferredDBusInit)

    def deferredDBusInit(self):
        self.usageIface = self.tryGetInterface('org.freesmartphone.ousaged',
                                               '/org/freesmartphone/Usage',
                                               'org.freesmartphone.Usage')
        if(self.usageIface):
            return False

        logging.info('entering deferred init for PanelPlugin')

        return True

    def configReader(self):
        self.panelConfig.read(self.configFiles)

        if (self.panelConfig.has_section('main')):
            if (self.panelConfig.has_option('main', 'plugins')):
                self.enabledIcons = self.panelConfig.get('main', 'plugins').replace(' ', '').split(',')
                
            if (self.panelConfig.has_option("main", "iconpath")):
                self.iconpath = self.panelConfig.get("main", "iconpath")

        for icon in self.enabledIcons:
            self.appendIcon(icon)

        self.iconOrder = self.enabledIcons[:]

        for icon in self.availableIcons:
            if (icon not in self.iconOrder):
                self.iconOrder.append(icon)

    def appendIcon(self, enabledIcon):
        if (enabledIcon == "battery"):
            from panelplugin.BatteryIcon import BatteryIcon
            self.icons.append(BatteryIcon(self, self.notificationAvailable))

        elif (enabledIcon == "brightness"):
            from panelplugin.BrightnessIcon import BrightnessIcon
            self.icons.append(BrightnessIcon(self, self.notificationAvailable))

        elif (enabledIcon == "bluetooth"):
            from panelplugin.BTIcon import BTIcon
            self.icons.append(BTIcon(self, self.notificationAvailable))

        elif (enabledIcon == "buttons"):
            from panelplugin.Buttons import Buttons
            self.icons.append(Buttons(self, self.notificationAvailable))

        elif (enabledIcon == "gps"):
            from panelplugin.GPSIcon import GPSIcon
            self.icons.append(GPSIcon(self, self.notificationAvailable))

        elif (enabledIcon == "gsm"):
            from panelplugin.GSMIcon import GSMIcon
            self.icons.append(GSMIcon(self, self.notificationAvailable))

        elif (enabledIcon == "keyboard"):
            from panelplugin.KeyboardIcon import KeyboardIcon
            self.icons.append(KeyboardIcon(self, self.notificationAvailable))

        elif (enabledIcon == "usb"):
            from panelplugin.USBIcon import USBIcon
            self.icons.append(USBIcon(self, self.notificationAvailable))

        elif (enabledIcon == "wifi"):
            from panelplugin.WiFiIcon import WiFiIcon
            self.icons.append(WiFiIcon(self, self.notificationAvailable))

        elif (enabledIcon == "screenlock"):
            from panelplugin.ScreenLockIcon import ScreenLockIcon
            self.screenlockicon =  ScreenLockIcon(self, self.notificationAvailable)
            self.icons.append(self.screenlockicon)

    def configWriter(self):
        if (not self.panelConfig.has_section("main")):
            self.panelConfig.add_section("main")

        self.panelConfig.set("main", "iconpath", self.iconpath)

        self.enabledIcons = []
        for icon in self.iconOrder:
            checkBox = self.configWindowActiveIconsCheckBoxes[icon]
            if (checkBox):
                if (checkBox.get_active()):
                    self.enabledIcons.append(icon)

        self.panelConfig.set("main", "plugins", ", ".join(self.enabledIcons))

        for icon in self.icons:
            icon.configWriter()

        try:
            configFile = file(os.path.expanduser("~/.panel-pluginrc"), "w")
            self.panelConfig.write(configFile)
            configFile.close()
        except:
            dialog = gtk.MessageDialog(
                                       parent         = None,
                                       flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
                                       type           = gtk.MESSAGE_INFO,
                                       buttons        = gtk.BUTTONS_OK,
                                       message_format = "Unable to write config!")

            dialog.set_title('Openmoko-Panel-Plugin')
            dialog.connect('response', self.show_destroy)
            dialog.show()

    def show_hide(self, widget, response_id, data = None):
        widget.destroy()

    def destroyer(self, widget, response_id, data = None):
        if (response_id == gtk.RESPONSE_OK):
            self.thread_terminate = True
            gtk.main_quit()

        else:
            widget.destroy()

    def popup(self, button, widget, data = None):
        dialog = gtk.MessageDialog(
                                   parent         = None,
                                   flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
                                   type           = gtk.MESSAGE_INFO,
                                   buttons        = gtk.BUTTONS_OK_CANCEL,
                                   message_format = "Do you want to close this Status Icon program?")
        dialog.set_title('Openmoko-Panel-Plugin')
        dialog.connect('response', self.destroyer)
        dialog.show()

    def updateIcons(self):
        # get's called from Buttons.py
        time.sleep(0.2) # really needed here ..

        if (not self.runningUpdate): # should do this with a mutex ..
            self.runningUpdate = True;

            for icon in self.icons:
                try:
                    icon.update()
                except:
                    logging.error('error while calling %s.update()' % icon.getName())

            self.runningUpdate = False;

        else:
            pass

        return not self.thread_terminate

    def iconEnabled(self, iconName):
        for icon in self.icons:
            if (iconName == icon.getName()):
                return icon

        return None

    def signalTo(self, selectedIconName = "", **kwargs):
        if (selectedIconName != ""):
            icon = self.iconEnabled(selectedIconName)

            if (icon):
                return icon.signal(**kwargs)

    def getName(self):
        return "PanelPlugin"

    def buttonHandler(self, name = "", action = "", seconds = 0):
        handled = False

        if (self.screenlockicon):
            handled = self.screenlockicon.buttonHandler(name, action, seconds)

        if (not handled):
            if (self.configWindowActive):
                if (name == "AUX"):
                    if (action == "pressed"):
                        self.pageShow = self.pageShow + 1 if (self.pageShow < self.pageCount) else 0
                        self.configWindowNotebook.set_current_page(self.pageShow)
            
                return True

            else:
                for icon in self.icons:
                    if (self.screenlockicon != icon):
                        handled = icon.buttonHandler(name, action, seconds, handled)

        return handled

    def configWindowCallback(self, widget, data = None):
        windowClose = False

        if (data == "cancel"):
            windowClose = True

        elif (data == "save"):
            windowClose = True

            self.iconpath = self.configWindowIconpathText.get_text()

            for icon in self.icons:
                icon.configWindowHandler(widget, data)

            self.configWriter()

        else:
            iconOrderChanged = False
            iconNumber = 0
            origIconOrder = self.iconOrder[:]

            for icon in self.iconOrder:
                if (data == "%s_up" % icon):
                    iconOrderChanged = True

                    tempIcon = origIconOrder[iconNumber - 1]
                    origIconOrder[iconNumber - 1] = origIconOrder[iconNumber]
                    origIconOrder[iconNumber] = tempIcon

                elif (data == "%s_dn" % icon):
                    iconOrderChanged = True
                
                    tempIcon = origIconOrder[iconNumber + 1]
                    origIconOrder[iconNumber + 1] = origIconOrder[iconNumber]
                    origIconOrder[iconNumber] = tempIcon

                iconNumber += 1

            if (iconOrderChanged):
                self.iconOrder = origIconOrder[:]

                for icon in self.configWindowVBoxIcons.get_children():
                    self.configWindowVBoxIcons.remove(icon)

                self.configWindowGetIconOrder()

                self.configWindow.show_all()

                for icon in self.iconOrder:
                    if (icon in self.enabledIcons):
                        self.signalTo(selectedIconName = icon, reorder = True)

        if (windowClose):
            self.configWindowDestroyer(None)

    def configWindowDestroyer(self, widget, data = None):
        self.configWindow.destroy()
        self.configWindowActive = False

    def configWindowDeleter(self, widget, event, data = None):
        return False

    def configWindowGetIconOrder(self):
        self.configWindowActiveIconsCheckBoxes = {}
        iconNumber = 0

        for icon in self.iconOrder:
            iconNumber += 1
            iconRow = gtk.HBox(False, 0)

            iconCheckBox = gtk.CheckButton("%s is Active" % icon)
            iconCheckBox.set_active(True if (icon in self.enabledIcons) else False)

            iconSpacer = gtk.Label()

            iconButtonUp = gtk.Button()
            iconButtonUpArrow = gtk.Arrow(gtk.ARROW_UP, gtk.SHADOW_OUT)
            iconButtonUp.add(iconButtonUpArrow)
            iconButtonUp.connect("clicked", self.configWindowCallback, "%s_up" % icon)
            iconButtonUp.set_sensitive(True if (iconNumber != 1) else False)

            iconButtonDn = gtk.Button()
            iconButtonDnArrow = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_OUT)
            iconButtonDn.add(iconButtonDnArrow)
            iconButtonDn.connect("clicked", self.configWindowCallback, "%s_dn" % icon)
            iconButtonDn.set_sensitive(True if (iconNumber != len(self.iconOrder)) else False)

            iconRow.pack_start(iconCheckBox, False)
            iconRow.pack_start(iconSpacer, True)
            iconRow.pack_start(iconButtonUp, False)
            iconRow.pack_start(iconButtonDn, False)

            self.configWindowActiveIconsCheckBoxes[icon] = iconCheckBox
            self.configWindowVBoxIcons.pack_start(iconRow, True, True, 2)

    def configWindowMaker(self, name = None):
        if (self.configWindowActive):
            self.configWindowDestroyer(None)
        else:
            self.configWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
            self.configWindow.set_title('Config')
            self.configWindow.set_border_width(5)
            self.configWindow.connect("delete_event", self.configWindowDeleter)
            self.configWindow.connect("destroy", self.configWindowDestroyer)
            self.configWindow.maximize()
            self.configWindow.show()

            self.configWindowFrame = gtk.Frame('Config (AUX cycles through tabs)')
            self.configWindow.add(self.configWindowFrame)

            self.configWindowVBox = gtk.VBox(False, 0)
            self.configWindowVBox.set_border_width(5)

            self.configWindowNotebook = gtk.Notebook()
            self.configWindowNotebook.set_tab_pos(gtk.POS_TOP)
            self.configWindowNotebook.set_scrollable(True)

            self.configWindowNotebookMainFrame = gtk.Frame()

            self.configWindowVBox2 = gtk.VBox(False, 0)
            self.configWindowNotebookMainFrame.add(self.configWindowVBox2)

            self.configWindowIconpathLabel = gtk.Label('Iconpath')
            self.configWindowIconpathLabel.set_justify(gtk.JUSTIFY_LEFT)

            self.configWindowIconpathText = gtk.Entry()
            self.configWindowIconpathText.set_text(self.iconpath)
            
            self.configWindowVBox2.pack_start(self.configWindowIconpathLabel)
            self.configWindowVBox2.pack_start(self.configWindowIconpathText)

            self.configWindowVBoxIcons = gtk.VBox(False, 0)

            self.configWindowGetIconOrder()

            self.configWindowVBox2.pack_start(self.configWindowVBoxIcons)

            self.configWindowChangesLabel = gtk.Label('(De-/Activations require restart!)')

            self.configWindowVBox2.pack_start(self.configWindowChangesLabel)

            self.configWindowNotebook.append_page(self.configWindowNotebookMainFrame, gtk.Label(self.getName()))

            self.pageCount = 0
            self.pageShow = 1

            for icon in self.icons:
                configFrame = icon.getConfigFrame()
                
                if (configFrame):
                    iconName = icon.getName()
                    self.configWindowNotebook.append_page(icon.getConfigFrame(), gtk.Label(iconName))

                    self.pageCount += 1

                    if (name == iconName):
                        self.pageShow = self.pageCount

            self.configWindowVBox.pack_start(self.configWindowNotebook)

            self.configWindowHBox = gtk.HBox(True, 0)

            self.configWindowCancelButton = gtk.Button('Cancel')
#            self.configWindowCancelButton.child.child.get_children()[1].modify_font(pango.FontDescription('sans bold 30'))
            self.configWindowCancelButton.child.modify_font(pango.FontDescription("sans bold 30"))
            self.configWindowCancelButton.set_border_width(5)
            self.configWindowCancelButton.connect('clicked', self.configWindowCallback, 'cancel')

            self.configWindowSaveButton = gtk.Button('Save')
#            self.configWindowSaveButton.child.child.get_children()[1].modify_font(pango.FontDescription('sans bold 30'))
            self.configWindowSaveButton.child.modify_font(pango.FontDescription("sans bold 30"))
            self.configWindowSaveButton.set_border_width(5)
            self.configWindowSaveButton.connect('clicked', self.configWindowCallback, 'save')

            self.configWindowHBox.pack_start(self.configWindowCancelButton)
            self.configWindowHBox.pack_start(self.configWindowSaveButton)
            
            self.configWindowVBox.pack_start(self.configWindowHBox)

            self.configWindowFrame.add(self.configWindowVBox)

            self.configWindow.show_all()

            self.configWindowNotebook.set_current_page(self.pageShow)

            self.configWindowActive = True

    def suspend(self):
        for icon in self.icons:
            icon.suspend()

        time.sleep(0.5)

        try:
            self.usageIface.Suspend()

        except Exception, e:
            logging.error('error while calling usageIface.Suspend()\n\t(%s)' % e)
            self.dbusReInit()

    def resume(self):
        gobject.threads_init()

        for icon in self.icons:
            icon.resume()

    def shutdown(self):
        try:
            self.usageIface.Shutdown()
        except Exception, e:
            logging.error('error while calling usageIface.Shutdown()\n\t(%s)' % e)

            self.dbusReInit()

    def reboot(self):
        try:
            self.usageIface.Reboot()
        except Exception, e:
            logging.error('error while calling usageIface.Reboot()\n\t(%s)' % e)

            self.dbusReInit()

    def tryGetInterface(self, busName, objectName, interfaceName):
        iface = None

        try:
            proxy = self.dbus.bus.get_object(busName, objectName)

        except DBusException, e:
            logging.error('could not create proxy for %s:%s\n\t(%s)' % (busName, objectName, e))
            proxy = None

        if (proxy):
            try:
                iface = Interface(proxy, interfaceName)

            except Exception, e:
                logging.error('could not create interface for %s:%s:%s\n\t(%s)' % (busName, objectName, interfaceName, e))

        return iface
