# vim: set fileencoding=utf-8 :

import gtk
import gobject
import logging

from panelplugin.StatusIcon import StatusIcon

#----------------------------------------------------------------------------#
class PowerStateIcon(StatusIcon):
#----------------------------------------------------------------------------#
    def __init__(self, ontext, offtext, dbusdevicename, parentObject, configName, notificationAvailable):
        self.parentObject = parentObject
        self.configName = configName
        self.dbusdevicename = dbusdevicename
        self.notificationAvailable = notificationAvailable

        self.dbusPowerSignal = None

        self.suspended = False
        self.powerstate = 0 

        StatusIcon.__init__(self, self.parentObject, self.configName)

        if (ontext):
            self.menuitem_on = gtk.MenuItem(ontext)
            self.menuitem_on.connect("activate", self.activateMenu, 'on')
            self.menuitem_on.show()
            self.menu.append(self.menuitem_on)

        if (offtext):
            self.menuitem_off = gtk.MenuItem(offtext)
            self.menuitem_off.connect("activate", self.activateMenu, 'off')
            self.menuitem_off.show()
            self.menu.append(self.menuitem_off)

    def dbusReInit(self, failure = False):
        self.dbusdeviceiface = None 
        StatusIcon.dbusReInit(self, failure)

    def deferredDBusInit(self):
        if (StatusIcon.deferredDBusInit(self)):
            return True

        logging.info('entering init for %s' % self.configName)

        self.dbusdeviceiface = self.parentObject.tryGetInterface('org.freesmartphone.odeviced',
                                                                 '/org/freesmartphone/Device/PowerControl/' + self.dbusdevicename,
                                                                 'org.freesmartphone.Device.PowerControl')
        
        if (self.dbusdeviceiface):
            if (self.dbusPowerSignal):
                try:
                    self.dbusPowerSignal.remove()
                    self.dbusPowerSignal = None
                except Exception, e:
                    logging.error("error while disconnecting from signal 'Power' \n\t(%s)" % e)

            if (not self.dbusPowerSignal):
                self.dbusPowerSignal = self.dbusdeviceiface.connect_to_signal("Power", self.updatePowerState)

            self.updatePowerState(self.dbusdeviceiface,self.getPowerState())

            return False

        logging.info('entering deferred init for %s' % self.configName)

        return True

    def activateMenu(self, widget, data = None):
        if (data == 'on'):
            self.preEnable()

            self.enable()

        elif (data == 'off'):
            self.preDisable()

            self.disable()

        StatusIcon.activateMenu(self, widget, data)

    def signal(self, **kwargs):
        if ('power' in kwargs):
            self.togglePowerState(kwargs['power'])

        elif ('enable' in kwargs):
            self.enable()

        elif ('disable' in kwargs):
            self.disable()

        elif ('getPower' in kwargs):
            return self.getPowerState()

        elif ('togglePower' in kwargs):
            self.togglePowerState(0 if (self.powerstate == 1) else 1)

            self.update()

            return self.powerstate

        else:
            return StatusIcon.signal(self, **kwargs)

    def getPowerState(self):
        state = 0

        try:
            state = self.dbusdeviceiface.GetPower()

        except Exception, e:
            logging.error('error while calling DeviceIface.GetPower() - assuming 0\n\t(%s)' % e)
            self.dbusReInit(True)

        return state

    def updatePowerState(self, name, status):
        if (status != self.powerstate):
            self.powerstate = status
            self.updateIcon()

            if (self.notificationAvailable & self.doNotification & (not self.suspended)):
                self.commitNotification()

            if (self.powerstate == 1):
                self.postEnable()
            else:
            	self.postDisable()

        self.suspended = False

    def preEnable(self):
        return

    def postEnable(self):
        return

    def preDisable(self):
        return

    def postDisable(self):
        return

    def togglePowerState(self, state):
        oldstate = self.powerstate

        try:
            self.dbusdeviceiface.SetPower(state)
            self.powerstate = state

        except Exception, e:
            logging.error('error while calling DeviceIface.SetPower(state) - not touching self.powerstate\n\t(%s)' % e)
            self.dbusReInit()
            self.powerstate = oldstate

    def enable(self):
        self.togglePowerState(1)

    def disable(self):
        self.togglePowerState(0)

    def suspend(self):
        StatusIcon.suspend(self)

        self.suspended = True
        self.doNotification = False
        self.suspend_powerstate = self.powerstate

        self.disable()

    def resume(self):
        self.togglePowerState(self.suspend_powerstate)
        
        StatusIcon.resume(self)
