# vim: set fileencoding=utf-8 :

import fcntl
import gobject
import gtk
import logging
import os
import pango
import termios

from panelplugin.StatusIcon import StatusIcon

#----------------------------------------------------------------------------#
class ScreenLockIcon(StatusIcon):
#----------------------------------------------------------------------------#
    def __init__(self, parentObject = None, notificationAvailable = False):
        self.parentObject = parentObject
        self.notificationAvailable = notificationAvailable

        self.touchDevice = "/dev/input/event1"
        self.fileHandle = None
        self.eviocgrab = 0x40044590

        self.activateLockScreenWindow = False
        self.autoReleaseOnCall = False

        self.autoRelockAfterCallDelay = 10

        self.lockWindowActive = False
        self.lockScreenWindowActive = False

        self.screenLocked = False
        self.screenLockRequested = False

        self.callStatus = ''

        self.dbusIdleSignal = None
        self.deviceIface = None

        self.idleState = ''
        self.idleStates = ["busy",
                           "idle",
                           "idle_dim",
                           "idle_prelock",
                           "lock",
                           "suspend",
                           "awake",
                           "none",
                           ]

        self.idleStateLock = 3
        self.idleStateOnManualLock = 3

        StatusIcon.__init__(self, self.parentObject, 'screenlock')

    def dbusReInit(self, failure = False):
        self.deviceIface = None
        StatusIcon.dbusReInit(self, failure)

    def deferredDBusInit(self):
        if (StatusIcon.deferredDBusInit(self)):
            return True

        logging.info('entering init for screenlock')

        self.deviceIface = self.parentObject.tryGetInterface('org.freesmartphone.odeviced',
                                                             '/org/freesmartphone/Device/IdleNotifier/0',
                                                             'org.freesmartphone.Device.IdleNotifier')

        if (self.deviceIface):
            if (self.dbusIdleSignal):
                try:
                    self.dbusIdleSignal.remove()
                    self.dbusIdleSignal = None
                except Exception, e:
                    logging.warning("error while disconnecting from signal 'State'\n\t(%s)" % e)

            if (not self.dbusIdleSignal):
                self.dbusIdleSignal = self.deviceIface.connect_to_signal("State", self.idleStateChanged)

            self.startupDone = True

            return False

        logging.info('entering deferred init for screenlock')

        return True

    def idleStateChanged(self, state = ''):
        logging.info("idlestate: %s" % state)

        if (state == self.idleStates[self.idleStateLock]):
            self.lockScreenCallback()

    def setIdleState(self, state = ''):
        try:
            self.deviceIface.SetState(state)

        except Exception, e:
            logging.error('error while setting idleState to %s\n\t(%s)' % (state, e))
            self.dbusReInit(True)

    def readTouchDevice(self, a, b):
        try:
            str = os.read(self.fileHandle, 16)

        except:
            return True

        if (not self.lockScreenWindowActive):
            pass

        else:
            pass

        return True

    def getIcon(self):
        return "%s/screen_lock_%s.png" % (self.parentObject.iconpath, 1 if self.screenLocked else 0)

    def configReader(self):
        if (StatusIcon.configReader(self)):
            if (self.config.has_option(self.configName, "autoReleaseOnCall")):
                self.autoReleaseOnCall = (self.config.get(self.configName, "autoReleaseOnCall").lower() == "true")

            if (self.config.has_option(self.configName, "activateLockScreenWindow")):
                self.activateLockScreenWindow = (self.config.get(self.configName, "activateLockScreenWindow").lower() == "true")

            if (self.config.has_option(self.configName, "autoLockOnIdleState")):
                try:
                    lockState = int(self.config.get(self.configName, "autoLockOnIdleState"))
                except:
                    lockState = len(self.idleStates) - 1

                if (lockState in self.idleStates):
                    self.idleStateLock = self.idleStates[lockState]

            if (self.config.has_option(self.configName, "idleStateOnManualLock")):
                try:
                    lockState = int(self.config.get(self.configName, "idleStateOnManualLock"))
                except:
                    lockState = len(self.idleStates) - 1

                if (lockState in self.idleStates):
                    self.idleStateOnManualLock = self.idleStates[lockState]

            return True

        return False

    def configWriter(self):
        StatusIcon.configWriter(self)

        self.config.set(self.configName, "autoReleaseOnCall", "True" if self.autoReleaseOnCall else "False")

        self.config.set(self.configName, "activateLockScreenWindow", "True" if self.activateLockScreenWindow else "False")

        self.config.set(self.configName, "autoLockOnIdleState", self.idleStates[self.idleStateLock])

        self.config.set(self.configName, "idleStateOnManualLock", self.idleStates[self.idleStateOnManualLock])

    def buttonHandler(self, name = "", action = "", seconds = 0, handled = False):
        if (not handled):
            if (self.screenLocked):
                if ((name in ["AUX", "POWER"]) and (action == "released") and (seconds == 0)):

                    if (name == 'POWER'):
                        self.screenLockRequested = False

                        self.releaseScreenCallback()

                return True

        return False

    def signal(self, **kwargs):
        if ('lockScreen' in kwargs):
            self.screenLockRequested = True

            self.lockScreenCallback()

        elif ('releaseScreen' in kwargs):
            self.screenLockRequested = False

            self.releaseScreenCallback()

        elif ('callStatus' in kwargs):
            self.callStatus = kwargs['callStatus']
            """
                incoming = The call is incoming (but not yet accepted),
                outgoing = The call is outgoing (but not yet established),
                active = The call is the active call (you can talk),
                held = The call is being held,
                release = The call has been released.            
            """            

            if (self.screenLocked):
                if (self.callStatus in ['incoming']):
                    if (self.autoReleaseOnCall):
                        self.releaseScreenCallback()

            else:
                if (self.callStatus in ['release']):
                    if (self.screenLockRequested):
                        if (self.autoRelockAfterCallDelay == 0):
                            self.lockScreenCallback()
                        else:
                            gobject.timeout_add_seconds(self.autoRelockAfterCallDelay, self.lockScreenCallback)

        else:
            return StatusIcon.signal(self, **kwargs)

    def activate(self, widget, data = None):
        if (self.lockWindowActive):
            self.menu.popup(None, None, None, 0, 0)
        else:
            self.lockWindowMaker()

    def lockScreenCallback(self):
        self.lockScreen()

        self.setIdleState(self.idleStates[self.idleStateOnManualLock])

        if (self.activateLockScreenWindow):
            self.lockScreenWindowMaker()

        return False

    def releaseScreenCallback(self):
        self.releaseScreen()

        if (self.lockScreenWindowActive):
            self.lockScreenWindowDestroyer(None)

        return False

    def lockScreen(self):
        if (not self.screenLocked):
            try:
                self.fileHandle = os.open(self.touchDevice, os.O_RDWR | os.O_NONBLOCK)

                if (fcntl.ioctl(self.fileHandle, self.eviocgrab, 1) != 0):
                    pass
                else:
                    self.eventWatcherTag = gobject.io_add_watch(self.fileHandle, gobject.IO_IN, self.readTouchDevice)
                    self.screenLocked = True

            except Exception, e:
                logging.error("error while requesting EVIOCGRAB for %s\n\t(%s)" % (self.touchDevice, e))
                self.screenLocked = False

        self.updateIcon()

    def releaseScreen(self):
        if (self.screenLocked):
            try:
                if (fcntl.ioctl(self.fileHandle, self.eviocgrab, 0) != 0):
                    pass

                else:
                    gobject.source_remove(self.eventWatcherTag)
                    self.screenLocked = False

                os.close(self.fileHandle)

            except Exception, e:
                logging.error("error while releasing EVIOCGRAB for %s\n\t(%s)" % (self.touchDevice, e))
                self.screenLocked = False

        self.updateIcon()

    def lockWindowDestroyer(self, widget, data = None):
        self.lockWindow.destroy()
        self.lockWindowActive = False

    def lockWindowDeleter(self, widget, event, data = None):
        return False

    def lockWindowCallback(self, widget, data = None):
        if (data == 'lockscreen'):
            self.lockWindowDestroyer(None)

            self.screenLockRequested = True

            self.lockScreenCallback()

        elif(data == 'cancel'):
            self.lockWindowDestroyer(None)

        self.lockWindowActive = False

    def lockWindowMaker(self):
        self.lockWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.lockWindow.set_title('Lock Screen')
        self.lockWindow.connect("delete_event", self.lockWindowDeleter)
        self.lockWindow.connect("destroy" , self.lockWindowDestroyer)
        self.lockWindow.maximize()
        self.lockWindow.show()

        self.lockWindowVBox = gtk.VBox(False, 0)
        self.lockWindow.add(self.lockWindowVBox)

        self.lockWindowLockButton = gtk.Button(' Lock\nScreen')
        self.lockWindowLockButton.child.modify_font(pango.FontDescription("sans bold 30"))
        self.lockWindowLockButton.set_border_width(5)
        self.lockWindowLockButton.connect('clicked', self.lockWindowCallback, 'lockscreen')

        self.lockWindowCancelButton = gtk.Button('Cancel')
#        self.lockWindowCancelButton.child.child.get_children()[1].modify_font(pango.FontDescription("sans bold 30"))
        self.lockWindowCancelButton.child.modify_font(pango.FontDescription("sans bold 30"))
        self.lockWindowCancelButton.set_border_width(5)
        self.lockWindowCancelButton.connect('clicked', self.lockWindowCallback, 'cancel')

        if (self.parentObject.kernel == '2.6.24'):
            self.lockWindowVBox.pack_start(gtk.Label('with 2.6.24-kernel\nEVIOCGRAB\nis broken'))

        self.lockWindowVBox.pack_start(self.lockWindowLockButton)
        self.lockWindowVBox.pack_start(self.lockWindowCancelButton, False)

        self.lockWindow.show_all()
        
        self.lockWindowActive = True

    def lockScreenWindowDestroyer(self, widget, data = None):
        self.lockScreenWindow.destroy()
        self.lockScreenWindowActive = False

    def lockScreenWindowDeleter(self, widget, event, data = None):
        return False

    def lockScreenWindowMaker(self):
        self.lockScreenWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.lockScreenWindow.connect("delete_event", self.lockScreenWindowDeleter)
        self.lockScreenWindow.connect("destroy" , self.lockScreenWindowDestroyer)
        self.lockScreenWindow.fullscreen()
        self.lockScreenWindow.show()

        if (self.parentObject.kernel == '2.6.24'):
            self.lockScreenWindowLabel = gtk.Label('with 2.6.24-kernel\nEVIOCGRAB\nis broken')

        else:
            self.lockScreenWindowLabel = gtk.Label(" Screen\n    is\n locked\n(release\n  with\n button)")
        
        self.lockScreenWindowLabel.modify_font(pango.FontDescription("sans bold 30"))

        self.lockScreenWindow.add(self.lockScreenWindowLabel)

        self.lockScreenWindow.show_all()

        self.lockScreenWindowActive = True

    def configWindowHandler(self, widget, data = None):
        self.activateLockScreenWindow = self.configCheckButtonLockScreenWindow.get_active()

        self.autoReleaseOnCall = self.configCheckButtonAutoReleaseOnCall.get_active()

        self.idleStateLock = self.autoLockOnIdleModeComboBox.get_active()

        self.idleStateOnManualLock = self.autoIdleModeOnManualLockComboBox.get_active()

        StatusIcon.configWindowHandler(self, widget, data)

    def getConfigFrame(self):
        configFrame = StatusIcon.getConfigFrame(self)

        self.configCheckButtonLockScreenWindow = gtk.CheckButton("Hide Desktop on Lock")
        self.configCheckButtonLockScreenWindow.set_active(self.activateLockScreenWindow)
        self.configCheckButtonLockScreenWindow.connect("clicked", self.configWindowHandler)

        self.configCheckButtonAutoReleaseOnCall = gtk.CheckButton("Auto-Release Lock on incoming Call")
        self.configCheckButtonAutoReleaseOnCall.set_active(self.autoReleaseOnCall)
        self.configCheckButtonAutoReleaseOnCall.connect("clicked", self.configWindowHandler)

        self.configScreenLockRow1 = gtk.HBox(False, 0)

        self.autoLockOnIdleModeComboBox = gtk.combo_box_new_text()

        for idleState in self.idleStates:
            self.autoLockOnIdleModeComboBox.append_text(idleState)

        self.autoLockOnIdleModeComboBox.set_active(self.idleStateLock)

        self.configScreenLockRow1.pack_start(gtk.Label("Auto-Lock on IdleState:"), False)
        self.configScreenLockRow1.pack_start(gtk.Label())
        self.configScreenLockRow1.pack_start(self.autoLockOnIdleModeComboBox, False)

        self.configScreenLockRow2 = gtk.HBox(False, 0)

        self.autoIdleModeOnManualLockComboBox = gtk.combo_box_new_text()

        for idleState in self.idleStates:
            self.autoIdleModeOnManualLockComboBox.append_text(idleState)

        self.autoIdleModeOnManualLockComboBox.set_active(self.idleStateOnManualLock)

        self.configScreenLockRow2.pack_start(gtk.Label("IdleState on Manual Lock:"), False)
        self.configScreenLockRow2.pack_start(gtk.Label())
        self.configScreenLockRow2.pack_start(self.autoIdleModeOnManualLockComboBox, False)

        self.configFrameVBox.pack_start(gtk.HSeparator(), False)
        self.configFrameVBox.pack_start(self.configCheckButtonLockScreenWindow, False)
        self.configFrameVBox.pack_start(self.configCheckButtonAutoReleaseOnCall, False)

        self.configFrameVBox.pack_start(gtk.HSeparator(), False)
        self.configFrameVBox.pack_start(self.configScreenLockRow1, False)
        self.configFrameVBox.pack_start(self.configScreenLockRow2, False)

        if (self.parentObject.kernel == '2.6.24'):
            self.configFrameVBox.pack_start(gtk.HSeparator(), False)
            self.configFrameVBox.pack_start(gtk.Label('with 2.6.24-kernel\nEVIOCGRAB\nis broken'))

        return configFrame
