# vim: set fileencoding=utf-8 :

import gobject
import gtk
import logging
import os
import pynotify

#----------------------------------------------------------------------------#
class StatusIcon(object):
#----------------------------------------------------------------------------#
    def __init__(self, parentObject, configName = ""):
        self.parentObject = parentObject
        self.configName = configName
    
        self.vibrating = False
        self.vibraId = 0

        self.updateInterval = 0

        self.visible = True # assume visible by default
        self.doNotification = True # assume we would send out notifications
        self.doVibrationOnNotification = False
        self.doVibrationOnActivation = False
        self.notificationIcon = False

        self.runPreSuspend = False
        self.preSuspendScript = ""
        self.runPostResume = False
        self.postResumeScript = ""

        if (self.configName != ""):
            self.config = self.parentObject.panelConfig
            self.configReader()

        self.icon = gtk.StatusIcon()
        self.icon.set_from_file(self.getIcon())
        self.icon.set_tooltip(self.getTooltip())
        self.icon.connect("activate", self.activate)
        self.icon.connect("popup_menu", self.popupMenu)
        self.icon.set_visible(self.visible)

        self.menu = gtk.Menu()
        self.menuItem_Name = gtk.MenuItem(configName)
        self.menuItem_Name.set_sensitive(False)
        self.menuItem_Name.show()
        
        self.menu.append(self.menuItem_Name)

        self.menuItem_Config = gtk.MenuItem("Config")
        self.menuItem_Config.connect("activate", self.activateMenu, 'config')
        self.menuItem_Config.show()
        self.menu.append(self.menuItem_Config)

        if (self.updateInterval != 0):
            gobject.timeout_add(self.updateInterval * 1000, self.update)

        self.startupDone = False

        gobject.timeout_add_seconds(10, self.deferredDBusInit)

    def dbusReInit(self, failure = False):
        logging.info('entering dbusReInit of %s' % self.configName)

        self.vibraIface = None

        if (failure):
            self.parentObject.spreadDBusFailure()
        
    def deferredDBusInit(self):
        self.vibraIface = self.parentObject.tryGetInterface('org.freesmartphone.odeviced',
                                                            '/org/freesmartphone/Device/LED/neo1973_vibrator',
                                                            'org.freesmartphone.Device.LED')

        if(self.vibraIface):
            return False

        logging.info('entering deferred init for VibraIface for %s' % self.configName)

        return True

    def configReader(self):
        if (self.config.has_section(self.configName)):
            if (self.config.has_option(self.configName, "updateinterval")):
                self.updateInterval = int(self.config.get(self.configName, "updateinterval"))

            if (self.config.has_option(self.configName, "visible")):
                self.visible = (self.config.get(self.configName, "visible").lower() == "true")

            if (self.config.has_option(self.configName, "notification")):
                self.doNotification = (self.config.get(self.configName, "notification").lower() == "true")

            if (self.config.has_option(self.configName, "notificationicon")):
                self.notificationIcon = (self.config.get(self.configName, "notificationicon").lower() == "true")

            if (self.config.has_option(self.configName, "vibration_on_notification")):
                self.doVibrationOnNotification = (self.config.get(self.configName, "vibration_on_notification").lower() == "true")
                
            if (self.config.has_option(self.configName, "vibration_on_activation")):
                self.doVibrationOnActivation = (self.config.get(self.configName, "vibration_on_activation").lower() == "true")

            if (self.config.has_option(self.configName, "runpresuspend")):
                self.runPreSuspend = (self.config.get(self.configName, "runpresuspend").lower() == "true")

            if (self.config.has_option(self.configName, "presuspendscript")):
                self.preSuspendScript = self.config.get(self.configName, "presuspendscript")

            if (self.config.has_option(self.configName, "runpostresume")):
                self.runPostResume = (self.config.get(self.configName, "runpostresume").lower() == "true")

            if (self.config.has_option(self.configName, "postresumescript")):
                self.postResumeScript = self.config.get(self.configName, "postresumescript")

            return True

        return False

    def configWriter(self):
        if (not self.config.has_section(self.configName)):
            self.config.add_section(self.configName)

        if (self.updateInterval == 0):
            try:
                self.config.remove_option(self.configName, "updateinterval")
            except:
                pass
        else:
            self.config.set(self.configName, "updateinterval", "%s" % self.updateInterval)

        self.config.set(self.configName, "visible", "True" if self.visible else "False")
        self.config.set(self.configName, "notification", "True" if self.doNotification else "False")

        if (self.doVibrationOnNotification):
            self.config.set(self.configName, "vibration_on_notification", "True")
        else:
            try:
                self.config.remove_option(self.configName, "vibration_on_notification")
            except:
                pass

        if (self.doVibrationOnActivation):
            self.config.set(self.configName, "vibration_on_activation", "True")
        else:
            try:
                self.config.remove_option(self.configName, "vibration_on_activation")
            except:
                pass

        if (self.notificationIcon):
            self.config.set(self.configName, "notificationicon", "True")
        else:
            try:
                self.config.remove_option(self.configName, "notificationicon")
            except:
                pass

        if (self.preSuspendScript == ""):
            try:
                self.config.remove_option(self.configName, "runpresuspend")
                self.config.remove_option(self.configName, "presuspendscript")
            except:
                pass                
        else:
            self.config.set(self.configName, "runpresuspend", "True" if self.runPreSuspend else "False")
            self.config.set(self.configName, "presuspendscript", self.preSuspendScript)
        

        if (self.postResumeScript == ""):
            try:
                self.config.remove_option(self.configName, "runpostresume")
                self.config.remove_option(self.configName, "postresumescript")
            except:
                pass
        else:
            self.config.set(self.configName, "runpostresume", "True" if self.runPostResume else "False")
            self.config.set(self.configName, "postresumescript", self.postResumeScript)

    def commitVibration(self, duration = 250):
        if (not self.vibraIface):
            return False

        if (not self.vibrating):
            self.vibrating = True

            self.vibraId = gobject.timeout_add(duration, self.commitVibration)

            try:
                self.vibraIface.SetBlinking(duration, 3000)

            except Exception, e:
                logging.error('error while trying to vibrate\n\t(%s)' % e)
                self.dbusReInit(True)

            return True

        else:
            self.vibrating = False

            try:
                self.vibraIface.SetBrightness(0)

            except Exception, e:
                logging.error('error while trying to vibrate\n\t(%s)' % e)
                self.dbusReInit(True)

            if (self.vibraId != 0):
                gobject.source_remove(self.vibraId)

            self.vibraId = 0

            return False

    def activate(self, widget, data = None):
        self.menu.popup(None, None, None, 0, 0)

    def activateMenu(self, widget, data = None):
        if (widget == self.menuItem_Config):
            self.parentObject.configWindowMaker(name = self.getName())

        else:
            if (self.doVibrationOnActivation):
                self.commitVibration(250)

        self.updateIcon()

    def popupMenu(self, **kwargs):
        pass

    def setVisible(self, show):
        self.visible = show
        self.icon.set_visible(self.visible)

    def getIcon(self):
        return ""

    def getTooltip(self):
        return ""

    def update(self):
        return self.updateIcon()
 
    def buttonHandler(self, name = "", action = "", seconds = 0, handled = False):
        return handled
 
    def signal(self, **kwargs):
        if ('update' in kwargs):
            return self.update()

        elif ('updateIcon' in kwargs):
            return self.updateIcon()

        elif ('reorder' in kwargs):
            self.icon.set_visible(False)
            
            self.icon.set_visible(self.visible)

            return True

    def updateIcon(self):
        self.icon.set_from_file(self.getIcon())
        self.icon.set_tooltip(self.getTooltip())
        
        return not self.parentObject.thread_terminate

    def commitNotification(self):
        if (self.doNotification & (not self.parentObject.starting)):
            if (self.notificationIcon):
                n = pynotify.Notification("Hardware", self.getNotificationText(), "file://%s" % self.getIcon())
            else:
                n = pynotify.Notification("Hardware", self.getNotificationText())

            n.set_urgency(pynotify.URGENCY_NORMAL)
            n.set_timeout(5000) # 10 seconds timeout
            n.show()

            if (self.doVibrationOnNotification & self.startupDone):
                self.commitVibration(250)

    def getNotificationText(self):
        return self.getTooltip()

    def getName(self):
        return self.configName

    def configWindowHandler(self, widget, data = None):
        if (self.configCheckButtonVisible.get_active()):
            self.setVisible(True)
        else:
            self.setVisible(False)

        self.updateInterval = int(self.configUpdateIntervalEntry.get_text())

        self.doNotification = self.configCheckButtonDoNotification.get_active()

        self.configCheckButtonDoVibrationOnNotification.set_sensitive(self.doNotification)

        self.doVibrationOnNotification = self.configCheckButtonDoVibrationOnNotification.get_active()

        self.notificationIcon = self.configCheckButtonNotificationIcon.get_active()

        self.configCheckButtonNotificationIcon.set_sensitive(self.doNotification)

        self.doVibrationOnActivation = self.configCheckButtonDoVibrationOnActivation.get_active()

        self.runPreSuspend = self.configCheckButtonRunPreSuspend.get_active()
        self.preSuspendScript = self.configRunPreSuspendEntry.get_text()
        self.configRunPreSuspendEntry.set_sensitive(self.runPreSuspend)

        self.runPostResume = self.configCheckButtonRunPostResume.get_active()
        self.postResumeScript = self.configRunPostResumeEntry.get_text()
        self.configRunPostResumeEntry.set_sensitive(self.runPostResume)

        return

    def getConfigFrame(self):
        configFrame = gtk.Frame()

        self.configScrolledWindow = gtk.ScrolledWindow()
        self.configScrolledWindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        configFrame.add(self.configScrolledWindow)

        self.configFrameVBox = gtk.VBox(False, 0)
        self.configScrolledWindow.add_with_viewport(self.configFrameVBox)

        self.configCheckButtonVisible = gtk.CheckButton("Visible")
        self.configCheckButtonVisible.set_active(self.visible)
        self.configCheckButtonVisible.connect("clicked", self.configWindowHandler)

        self.configHBox1 = gtk.HBox(False, 0)

        self.configUpdateIntervalEntry = gtk.Entry()
        self.configUpdateIntervalEntry.set_text("%s" % self.updateInterval)

        self.configUpdateIntervalLabel = gtk.Label('Update Interval (secs - 0 = off)')

        self.configHBox1.pack_start(self.configUpdateIntervalEntry, False)
        self.configHBox1.pack_start(self.configUpdateIntervalLabel)

        self.configHBox2 = gtk.HBox(True, 0)

        self.configCheckButtonDoNotification = gtk.CheckButton("Send Notification")
        self.configCheckButtonDoNotification.set_active(self.doNotification)
        self.configCheckButtonDoNotification.connect("clicked", self.configWindowHandler)

        self.configCheckButtonDoVibrationOnNotification = gtk.CheckButton("Vibrate on Notification")
        self.configCheckButtonDoVibrationOnNotification.set_active(self.doVibrationOnNotification)
        self.configCheckButtonDoVibrationOnNotification.set_sensitive(self.doNotification)
        self.configCheckButtonDoVibrationOnNotification.connect("clicked", self.configWindowHandler)

        self.configCheckButtonNotificationIcon = gtk.CheckButton("Show Icon on Notification")
        self.configCheckButtonNotificationIcon.set_active(self.notificationIcon)
        self.configCheckButtonNotificationIcon.set_sensitive(self.doNotification)
        self.configCheckButtonNotificationIcon.connect("clicked", self.configWindowHandler)

        self.configHBox2.pack_start(self.configCheckButtonDoNotification)
        self.configHBox2.pack_start(self.configCheckButtonDoVibrationOnNotification)

        self.configCheckButtonDoVibrationOnActivation = gtk.CheckButton("Vibrate on Activation")
        self.configCheckButtonDoVibrationOnActivation.set_active(self.doVibrationOnActivation)
        self.configCheckButtonDoVibrationOnActivation.connect("clicked", self.configWindowHandler)

        self.configHBox3 = gtk.HBox(True, 0)

        self.configCheckButtonRunPreSuspend = gtk.CheckButton("Run Pre-Suspend")
        self.configCheckButtonRunPreSuspend.set_active(self.runPreSuspend)
        self.configCheckButtonRunPreSuspend.connect("clicked", self.configWindowHandler)

        self.configRunPreSuspendEntry = gtk.Entry()
        self.configRunPreSuspendEntry.set_text(self.preSuspendScript)
        self.configRunPreSuspendEntry.set_sensitive(self.runPreSuspend)

        self.configHBox3.pack_start(self.configCheckButtonRunPreSuspend)
        self.configHBox3.pack_start(self.configRunPreSuspendEntry)

        self.configHBox4 = gtk.HBox(True, 0)

        self.configCheckButtonRunPostResume = gtk.CheckButton("Run Post-Resume")
        self.configCheckButtonRunPostResume.set_active(self.runPostResume)
        self.configCheckButtonRunPostResume.connect("clicked", self.configWindowHandler)

        self.configRunPostResumeEntry = gtk.Entry()
        self.configRunPostResumeEntry.set_text(self.postResumeScript)
        self.configRunPostResumeEntry.set_sensitive(self.runPostResume)

        self.configHBox4.pack_start(self.configCheckButtonRunPostResume, False)
        self.configHBox4.pack_start(self.configRunPostResumeEntry, False)

        self.configFrameVBox.pack_start(self.configCheckButtonVisible, False)

        self.configFrameVBox.pack_start(gtk.HSeparator(), False)
        self.configFrameVBox.pack_start(self.configHBox1, False)

        self.configFrameVBox.pack_start(gtk.HSeparator(), False)
        self.configFrameVBox.pack_start(self.configHBox2, False)
        self.configFrameVBox.pack_start(self.configCheckButtonDoVibrationOnActivation, False)
        self.configFrameVBox.pack_start(self.configCheckButtonNotificationIcon, False)

        self.configFrameVBox.pack_start(gtk.HSeparator(), False)
        self.configFrameVBox.pack_start(self.configHBox3, False)
        self.configFrameVBox.pack_start(self.configHBox4, False)

        return configFrame

    def suspend(self):
        if (self.runPreSuspend & (self.preSuspendScript != "")):
            os.system("%s &" % self.preSuspendScript)

    def resume(self):
        self.update()

        if (self.runPostResume & (self.postResumeScript != "")):
            os.system("%s &" % self.postResumeScript)
