# vim: set fileencoding=utf-8 :

import gtk
import pango
from panelplugin.PowerStateIcon import PowerStateIcon

#----------------------------------------------------------------------------#
class USBIcon(PowerStateIcon):
#----------------------------------------------------------------------------#
    def __init__(self, parentObject = None, notificationAvailable = False):
        self.parentObject = parentObject
        self.notificationAvailable = notificationAvailable

        self.usbWindowActive = False
        self.usbMode = 'device' # default - update later
        self.usbGadget = 'net'  # default - update later

        PowerStateIcon.__init__(self,
                                None,
                                None,
                                "0",
                                self.parentObject,
                                "usb",
                                self.notificationAvailable)

        self.usbMode = 'host' if self.powerstate else 'device'
        self.dbusReInit()

    def windowCallback(self, widget, data):
        if (data == 'host'):
            self.usbModeButtonDevice.set_active(False)
            self.usbMode = 'host'
            self.dbusdeviceiface.SetPower(1)

        elif (data == 'device'):
            self.usbModeButtonHost.set_active(False)
            self.usbMode = 'device'
            self.dbusdeviceiface.SetPower(0)

        elif (data == "drive"):
            self.usbGadgetButtonNet.set_active(False)
            self.usbGadget = "drive";

        elif (data == "net"):
            self.usbGadgetButtonDrive.set_active(False)
            self.usbGadget = "net";

        elif (data == 'close'):
            self.usbWindowActive = False
            self.usbWindow.destroy()
            self.usbWinodw = None

        self.usbGadgetButtonDrive.set_sensitive(self.usbMode == 'device')
        self.usbGadgetButtonNet.set_sensitive(self.usbMode == 'device')

    def activate(self, widget, data = None):
        if self.usbWindowActive:
            self.usbWindow.destroy()
            self.usbWindow = None

        else:
            self.usbWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
            self.usbWindow.set_border_width(10)
            self.usbWindow.set_title('USB-Mode')
            self.usbWindow.maximize()

            self.usbWindowFrame = gtk.Frame('USB-Mode')
            self.usbWindow.add(self.usbWindowFrame)

            self.usbWindowVBox = gtk.VBox(False, 0)
            self.usbWindowVBox.set_border_width(10)
            self.usbWindowFrame.add(self.usbWindowVBox)

            self.usbWindowHBoxMode = gtk.HBox(True, 0)
            self.usbWindowVBox.add(self.usbWindowHBoxMode)

            self.usbModeButtonHost = gtk.ToggleButton('Host')
            self.usbModeButtonHost.child.modify_font(pango.FontDescription("sans bold 30"))
            self.usbModeButtonHost.connect("toggled", self.windowCallback, 'host')
            self.usbModeButtonHost.show()

            self.usbModeButtonDevice = gtk.ToggleButton('Device')
            self.usbModeButtonDevice.child.modify_font(pango.FontDescription("sans bold 30"))
            self.usbModeButtonDevice.connect("toggled", self.windowCallback, 'device')
            self.usbModeButtonDevice.show()

            self.usbWindowHBoxMode.pack_start(self.usbModeButtonHost)
            self.usbWindowHBoxMode.pack_start(self.usbModeButtonDevice)
            self.usbWindowHBoxMode.show()

            self.usbWindowHBoxGadget = gtk.HBox(True, 0)
            self.usbWindowVBox.add(self.usbWindowHBoxGadget)

            self.usbGadgetButtonDrive = gtk.ToggleButton('Drive')
            self.usbGadgetButtonDrive.child.modify_font(pango.FontDescription("sans bold 30"))
            self.usbGadgetButtonDrive.connect('toggled', self.windowCallback, 'drive')
            self.usbGadgetButtonDrive.set_sensitive(self.usbMode == 'device')
            self.usbGadgetButtonDrive.show()

            self.usbGadgetButtonNet = gtk.ToggleButton('Net')
            self.usbGadgetButtonNet.child.modify_font(pango.FontDescription("sans bold 30"))
            self.usbGadgetButtonNet.connect('toggled', self.windowCallback, 'net')
            self.usbGadgetButtonNet.set_sensitive(self.usbMode == 'device')
            self.usbGadgetButtonNet.show()

            self.usbWindowHBoxGadget.pack_start(self.usbGadgetButtonDrive)
            self.usbWindowHBoxGadget.pack_start(self.usbGadgetButtonNet)
            self.usbWindowHBoxGadget.show()

            self.usbWindowButtonOK = gtk.Button(label='Close')
            self.usbWindowButtonOK.child.modify_font(pango.FontDescription("sans bold 30"))
            self.usbWindowButtonOK.connect('clicked', self.windowCallback, 'close')
            self.usbWindowButtonOK.show()

            self.usbWindowVBox.pack_start(self.usbWindowButtonOK)
            self.usbWindowVBox.show()

            self.usbGadgetButtonDrive.set_active(self.usbGadget == 'drive')
            self.usbGadgetButtonNet.set_active(self.usbGadget == 'net')
            self.usbModeButtonHost.set_active(self.usbMode == 'host')
            self.usbModeButtonDevice.set_active(self.usbMode == 'device')

            self.usbWindowFrame.show()
            self.usbWindow.show()

        self.usbWindowActive = not self.usbWindowActive

    def getTooltip(self):
        if (self.usbMode == "device"):
            if (self.usbGadget == "net"):
                tooltip = "Device: Net";

            elif (self.usbGadget == "drive"):
                tooltip = "Device: Drive";

        else:
            tooltip = "Host";

        return tooltip

    def getIcon(self):
        icon = ''
        self.usbMode = 'host' if self.powerstate else 'device'

        if (self.usbMode == "device"):
            if (self.usbGadget == "net"):
                icon = "%s/usb_net.png" % self.parentObject.iconpath

            elif (self.usbGadget == "drive"):
                icon = "%s/usb_drive.png" % self.parentObject.iconpath
        else:
            icon = "%s/usb_host.png" % self.parentObject.iconpath

        return icon
