# vim: set fileencoding=utf-8 :

import gtk
import gobject
import logging
import os
import time

from panelplugin.StatusIcon import StatusIcon

#----------------------------------------------------------------------------#
class UsageIcon(StatusIcon):
#----------------------------------------------------------------------------#
    def __init__(self, ontext, offtext, resourceName, parentObject, configName, notificationAvailable):
        self.resourceName = resourceName
        self.parentObject = parentObject
        self.configName = configName
        self.notificatonAvailable = notificationAvailable

        self.resourceRequested = False

        self.runPostEnable = False
        self.postEnableScript = ""
        self.runPreDisable = False
        self.preDisableScript = ""

        self.runPostDisable = False
        self.postDisableScript = ""
        self.runPreEnable = False
        self.preEnableScript = ""

        self.resourceState = 0
        self.resourceAttributes = {}

        self.dbusResourceChangedSignal = None
        self.dbusResourceAvailableSignal = None

        self.usageIface = None

        StatusIcon.__init__(self, self.parentObject, self.configName)

        self.menuitem_on = gtk.MenuItem(ontext)
        self.menuitem_on.connect("activate", self.activateMenu)
        self.menuitem_on.show()

        self.menuitem_off = gtk.MenuItem(offtext)
        self.menuitem_off.connect("activate", self.activateMenu)
        self.menuitem_off.show()

        self.menu.append(self.menuitem_on)
        self.menu.append(self.menuitem_off)

    def dbusReInit(self, failure = False):
        self.usageIface = None
        StatusIcon.dbusReInit(self, failure)

    def deferredDBusInit(self):
        if (StatusIcon.deferredDBusInit(self)):
            return True

        logging.info('entering init for UsageInterface for %s' % self.configName)

        self.usageIface = self.parentObject.tryGetInterface('org.freesmartphone.ousaged',
                                                            '/org/freesmartphone/Usage',
                                                            'org.freesmartphone.Usage')

        if (self.usageIface):
            if (self.dbusResourceChangedSignal):
                try:
                    self.dbusResourceChangedSignal.remove()
                    self.dbusResourceChangedSignal = None

                except Exception, e:
                    logging.error("error while disconnecting from signal 'ResourceChanged'\n\t(%s)" % e)

            if (not self.dbusResourceChangedSignal):
                self.dbusResourceChangedSignal = self.usageIface.connect_to_signal('ResourceChanged', self.resourceChanged)

            if (self.dbusResourceAvailableSignal):
                try:
                    self.dbusResourceAvailableSignal.remove()
                    self.dbusResourceAvailableSignal = None

                except Exception, e:
                    logging.error("error while disconnecting from signal 'ResourceAvailable'\n\t(%s)" % e)

            if (not self.dbusResourceAvailableSignal):
                self.dbusResourceAvailableSignal = self.usageIface.connect_to_signal('ResourceAvailable', self.resourceAvailable)

            self.resourceState = -1
            self.resourceChanged(self.resourceName, self.getResourceState(), None)

            self.startupDone = True

            return False

        logging.info('entering deferred init for UsageInterface for %s' % self.configName)

        return True

    def configReader(self):
        if (StatusIcon.configReader(self)):
            if (self.config.has_option(self.configName, "runpostenable")):
                self.runPostEnable = (self.config.get(self.configName, "runpostenable").lower() == "true")

            if (self.config.has_option(self.configName, "postenablescript")):
                self.postEnableScript = self.config.get(self.configName, "postenablescript")

            if (self.config.has_option(self.configName, "runpredisable")):
                self.runPreDisable = (self.config.get(self.configName, "runpredisable").lower() == "true")

            if (self.config.has_option(self.configName, "predisablescript")):
                self.preDisableScript = self.config.get(self.configName, "predisablescript")

            if (self.config.has_option(self.configName, "runpostdisable")):
                self.runPostDisable = (self.config.get(self.configName, "runpostdisable").lower() == "true")

            if (self.config.has_option(self.configName, "postdisablescript")):
                self.postDisableScript = self.config.get(self.configName, "postdisablescript")

            if (self.config.has_option(self.configName, "runpreenable")):
                self.runPreEnable = (self.config.get(self.configName, "runpreenable").lower() == "true")

            if (self.config.has_option(self.configName, "preenablescript")):
                self.preEnableScript = self.config.get(self.configName, "preenablescript")

            return True

        return False

    def configWriter(self):
        StatusIcon.configWriter(self)

        if (self.preDisableScript == ""):
            try:
                self.config.remove_option(self.configName, "runpredisable")
                self.config.remove_option(self.configName, "predisablescript")
            except:
                pass
        else:
            self.config.set(self.configName, "runpredisable", "True" if self.runPreDisable else "False")
            self.config.set(self.configName, "predisablescript", self.preDisableScript)

        if (self.postEnableScript == ""):
            try:
                self.config.remove_option(self.configName, "runpostenable")
                self.config.remove_option(self.configName, "postenablescript")
            except:
                pass
        else:
            self.config.set(self.configName, "runpostenable", "True" if self.runPostEnable else "False")
            self.config.set(self.configName, "postenablescript", self.postEnableScript)

        if (self.preEnableScript == ""):
            try:
                self.config.remove_option(self.configName, "runpreenable")
                self.config.remove_option(self.configName, "preenablescript")
            except:
                pass
        else:
            self.config.set(self.configName, "runpreenable", "True" if self.runPreEnable else "False")
            self.config.set(self.configName, "preenablescript", self.preEnableScript)

        if (self.postDisableScript == ""):
            try:
                self.config.remove_option(self.configName, "runpostdisable")
                self.config.remove_option(self.configName, "postdisablescript")
            except:
                pass
        else:
            self.config.set(self.configName, "runpostdisable", "True" if self.runPostDisable else "False")
            self.config.set(self.configName, "postdisablescript", self.postDisableScript)

    def getTooltip(self):
        tooltip = "%s is %s" % (self.resourceName, "On" if (self.resourceState == 1) else "Off")

        users = self.getResourceUsers()

        if (users):
            tooltip += " (%s Users)" % len(users)

        return tooltip

    def getNotificationText(self):
        return "%s resourceState changed" % self.resourceName

    def activateMenu(self, widget, data = None):
        if (widget == self.menuitem_on):
            if (not self.resourceRequested):
                self.requestResource()

        elif (widget == self.menuitem_off):
            if (self.resourceRequested):
                self.releaseResource()

        StatusIcon.activateMenu(self, widget, data)

    def signal(self, **kwargs):
        if ('togglePower' in kwargs):
            if (self.resourceRequested):
                self.releaseResource()
            else:
                self.requestResource()

        elif ('getPower' in kwargs):
            return self.resourceRequested

        else:
            return StatusIcon.signal(self, **kwargs)

    def isResourceInList(self):
        if (self.usageIface):
            try:
                resourceList = self.usageIface.ListResources()

            except Exception, e:
                logging.error('error while requesting ResourceList - assuming empty list\n\t(%s)' % e)
                resourceList = {}
                self.dbusReInit(True)

            if (self.resourceName in resourceList):
                return True

        return False

    def getResourceUsers(self):
        resourceUsers = None

        if (self.isResourceInList()):
            try:
                resourceUsers = self.usageIface.GetResourceUsers(self.resourceName)

            except Exception, e:
                logging.error('error while requesting GetResourceUsers for %s - assuming empty list\n\t(%s)' % (self.resourceName, e))
                self.dbusReInit()

        return resourceUsers

    def getResourceState(self):
        state = 0

        if (self.isResourceInList()):
            try:
                state = self.usageIface.GetResourceState(self.resourceName)

            except Exception, e:
                logging.error('error while requesting ResourceState for %s - assuming 0\n\t(%s)' % (self.resourceName, e))
                self.dbusReInit()
        
        return state

    def requestResource(self):
        if (self.isResourceInList()):
            self.preEnable()

            try:
                self.usageIface.RequestResource(self.resourceName)
                self.resourceRequested = True

            except Exception, e:
                logging.error('error while requesting resource %s\n\t(%s)' % (self.resourceName, e))
                self.dbusReInit()

    def releaseResource(self):
        if (self.isResourceInList()):
            self.preDisable()

            try:
                self.usageIface.ReleaseResource(self.resourceName)
                self.resourceRequested = False

            except Exception, e:
                logging.error('error while releasing resource %s\n\t(%s)' % (self.resourceName, e))
                self.dbusReInit()

    def resourceChanged(self, name, state, attributes):
        if (name == self.resourceName):
            logging.info("resourceChanged: name: %s state: %s attributes: %s" % (name, state, attributes))

            if ((self.resourceState != state) or (self.resourceAttributes != attributes)):
                self.resourceState = state
                self.resourceAttributes = attributes

                if (self.resourceState):
                    self.postEnable()

                else:
                    self.postDisable()

                self.updateIcon()

                if (self.notificationAvailable and self.doNotification):
                    self.commitNotification()

    def resourceAvailable(self, name, available):
        if (name == self.resourceName):
            logging.info("resourceAvailable: name: %s available: %s" % (name, available))
            self.available = available

            if (available):
                self.resourceState = self.getResourceState()

                if (self.resourceRequested):
                    self.requestResource()

            else:
                self.resourceState = 0

            self.updateIcon()

    def postEnable(self):
        logging.info("entering postEnable of %s" % self.configName)
        if (self.runPostEnable & (self.postEnableScript != "")):
            os.system("%s &" % self.postEnableScript)

    def preEnable(self):
        logging.info("entering preEnable of %s" % self.configName)
        if (self.runPreEnable & (self.preEnableScript != "")):
            os.system(self.preEnableScript)

    def postDisable(self):
        logging.info("entering postDisable of %s" % self.configName)
        if (self.runPostDisable & (self.postDisableScript != "")):
            os.system("%s &" % self.postDisableScript)

    def preDisable(self):
        logging.info("entering preDisable of %s" % self.configName)
        if (self.runPreDisable & (self.preDisableScript !="")):
            os.system(self.preDisableScript)

    def configWindowHandler(self, widget, data = None):
        self.runPreDisable = self.configCheckButtonRunPreDisable.get_active()
        self.preDisableScript = self.configRunPreDisableEntry.get_text()
        self.configRunPreDisableEntry.set_sensitive(self.runPreDisable)

        self.runPostEnable = self.configCheckButtonRunPostEnable.get_active()
        self.postEnableScript = self.configRunPostEnableEntry.get_text()
        self.configRunPostEnableEntry.set_sensitive(self.runPostEnable)

        self.runPreEnable = self.configCheckButtonRunPreEnable.get_active()
        self.preEnableScript = self.configRunPreEnableEntry.get_text()
        self.configRunPreEnableEntry.set_sensitive(self.runPreEnable)

        self.runPostDisable = self.configCheckButtonRunPostDisable.get_active()
        self.postDisableScript = self.configRunPostDisableEntry.get_text()
        self.configRunPostDisableEntry.set_sensitive(self.runPostDisable)

        StatusIcon.configWindowHandler(self, widget, data)

    def getConfigFrame(self):
        configFrame = StatusIcon.getConfigFrame(self)

        self.configHBox5 = gtk.HBox(True, 0)

        self.configCheckButtonRunPreEnable = gtk.CheckButton("Run Pre-Enable")
        self.configCheckButtonRunPreEnable.set_active(self.runPreEnable)
        self.configCheckButtonRunPreEnable.connect("clicked", self.configWindowHandler)

        self.configRunPreEnableEntry = gtk.Entry()
        self.configRunPreEnableEntry.set_text(self.preEnableScript)
        self.configRunPreEnableEntry.set_sensitive(self.runPreEnable)

        self.configHBox5.pack_start(self.configCheckButtonRunPreEnable, False)
        self.configHBox5.pack_start(self.configRunPreEnableEntry)

        self.configHBox6 = gtk.HBox(True, 0)

        self.configCheckButtonRunPostEnable = gtk.CheckButton("Run Post-Enable")
        self.configCheckButtonRunPostEnable.set_active(self.runPostEnable)
        self.configCheckButtonRunPostEnable.connect("clicked", self.configWindowHandler)

        self.configRunPostEnableEntry = gtk.Entry()
        self.configRunPostEnableEntry.set_text(self.postEnableScript)
        self.configRunPostEnableEntry.set_sensitive(self.runPostEnable)

        self.configHBox6.pack_start(self.configCheckButtonRunPostEnable, False)
        self.configHBox6.pack_start(self.configRunPostEnableEntry)

        self.configHBox7 = gtk.HBox(True, 0)

        self.configCheckButtonRunPreDisable = gtk.CheckButton("Run Pre-Disable")
        self.configCheckButtonRunPreDisable.set_active(self.runPreDisable)
        self.configCheckButtonRunPreDisable.connect("clicked", self.configWindowHandler)

        self.configRunPreDisableEntry = gtk.Entry()
        self.configRunPreDisableEntry.set_text(self.preDisableScript)
        self.configRunPreDisableEntry.set_sensitive(self.runPreDisable)

        self.configHBox7.pack_start(self.configCheckButtonRunPreDisable, False)
        self.configHBox7.pack_start(self.configRunPreDisableEntry)

        self.configHBox8 = gtk.HBox(True, 0)

        self.configCheckButtonRunPostDisable = gtk.CheckButton("Run Post-Disable")
        self.configCheckButtonRunPostDisable.set_active(self.runPostDisable)
        self.configCheckButtonRunPostDisable.connect("clicked", self.configWindowHandler)

        self.configRunPostDisableEntry = gtk.Entry()
        self.configRunPostDisableEntry.set_text(self.postDisableScript)
        self.configRunPostDisableEntry.set_sensitive(self.runPostDisable)

        self.configHBox8.pack_start(self.configCheckButtonRunPostDisable, False)
        self.configHBox8.pack_start(self.configRunPostDisableEntry)

        self.configFrameVBox.pack_start(gtk.HSeparator(), False)

        self.configFrameVBox.pack_start(self.configHBox5, False)
        self.configFrameVBox.pack_start(self.configHBox6, False)

        self.configFrameVBox.pack_start(gtk.HSeparator(), False)

        self.configFrameVBox.pack_start(self.configHBox7, False)
        self.configFrameVBox.pack_start(self.configHBox8, False)

        return configFrame
